%{
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#include "typedef.h"
#include "ytab.h"
extern int Debug;
extern 
char* InitString( char* );
extern void yyerror( const char* );

/*
%p 1200
*/


%}

%e 700
%n 350
%p 1200
%k 550
%a 1300
%o 1500

blancs		[ \t\n]*
digits		[0-9]+
digit		[0-9]
maj		[A-Z]

tag		({digit}|{maj})
indic1		("I1"|"i1")
indic2		("I2"|"i2")
subfield	"$"[0-9a-zA-Z]
fixedpos	"/"{digits}("-"{digits})?"/"	

%%
\'[^\']+\'	{ 
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=0;
		  yylval.inst->str=InitString(yytext);
		  return STRING;
		}
\'\'		{
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=0;
		  yylval.inst->str=strdup("");
		  return STRING;
		}
{tag}{tag}{tag}/("("|"$"|"/"|{indic1}|{indic2})	{
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=0;
		  yylval.inst->str=(char*)strdup(yytext);
		  return TAG;
		}
"@"{tag}{tag}{tag}	{
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=0;
		  yylval.inst->str=(char*)strdup(yytext+1);
		  return TAG;
		}
{indic1}	{ return I1; }
{indic2}	{ return I2; }
{subfield}	{ 
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=0;
		  yylval.inst->str=(char*)strdup(yytext);
		  return STAG;
		}
{fixedpos}	{
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=0;
		  yylval.inst->str=(char*)strdup(yytext);
		  return FIX;
		}
"#"		{ return FIN; }
[,{}()]		return *yytext;
";"		{ return SEP; }
{digits}	{
		  yylval.inst=(TypeInst*)malloc(sizeof(TypeInst));
		  yylval.inst->val=atoi(yytext);
		  yylval.inst->str=NULL;
		  return NUMERIC;
		}
"+"			return PLUS;
"-"			return MOINS;
"*"			return MULTIPLIE;
":"			return DIVISE;
S			return VARS;
[N|n][T|t][O|o]		return VAR_NTO;
[N|n][S|s][O|o]		return VAR_NSO;
[N|n][T|t]		return VAR_NT;
[N|n][S|s]		return VAR_NS;
[N|n][O|o]		return VAR_NO;
[N|n]			return VAR_N;
[I|i]f			return IF;
[T|t]hen		return THEN;
[E|e]lse		return ELSE;
[A|a]nd			return AND;
[O|o]r			return OR;
[N|n]ot			return NOT;
"!="			return NE;
"="			return EQ;
[I|i][N|n]		return IN;
">="			return GE;
"<="			return LE;
">"			return GT;
"<"			return LT;
[S|s]tr			return STR;
[V|v]al			return VAL;
[L|l]en			return LEN;
[S|s]to			return STO;
[M|m]em			return MEM;
[E|e]xc			return EXC;
[C|c]lr			return CLR;
[F|f]rom		return FROM;
[T|t]o			return TO;
[B|b]etween		return BETWEEN;
[S|s]trict		return STRICT;
[D|d]elete		return DELETE;
[R|r]eplace		return REPLACE;
[R|r]eplace[O|o]cc return REPLACEOCC;
[A|a]t			return AT;
[B|b]y			return BY;
[B|b]eginning		return BEGINING;
[E|e]nding		return END;
[B|b]oundaries		return BOTH;
[B|b][F|f]irst		return BFIRST;
[E|e][F|f]irst		return EFIRST;
[B|b][L|l]ast		return BLAST;
[E|e][L|l]ast		return ELAST;
[N|n]umber		return WNUMBER;
[S|s]tring		return WSTRING;
[E|e]xists		return EXISTS;
[P|p]recedes		return PRECEDES;
[F|f]ollows		return FOLLOWS;
[R|r]edo		return REDO;
[S|s]ort		return SORT;
[N|n]ext		return NEXT;
[L|l]ast		return LAST;
[T|t]able		return TABLE;
[U|u]pper		return UPPER;
[L|l]ower		return LOWER;
[Y|y]ear		return YEAR;
[M|m]onth		return MONTH;
[D|d]ay			return DAY;
[H|h]our		return HOUR;
[M|m]inute		return MINUTE;
[S|s]econd		return SECOND;
[O|o]rdinal		return ORDINAL;
[N|n]ext[S|s]ub		return NEXTBAL;
[P|p]revious[S|s]ub	return PREVIOUSBAL;
{blancs}			;
.			yyerror("lex error");
%%

char* InitString( char* t )
{
	int i=1, j=0;
	int d,u;
	char* rc=(char*)malloc(strlen(t));

	while( t[i] )
	{
		switch(t[i])
		{
		case '$' : 
			if (t[i+1]=='$')
				rc[j++]=t[++i];
			else
				rc[j++]=0x1F; 
			++i; 
			break;
		case '\\' :
			++i;
			if (t[i]=='\\' || t[i]=='$') rc[j++]=t[i++];
			else
			{
				d=t[i++];
				u=t[i++];
				if (d>='0' && d<='9') d-='0';
				else
				if (d>='A' && d<='F') d-='A'-10;
				else
				if (d>='a' && d<='f') d-='a'-10;
				if (u>='0' && u<='9') u-='0';
				else
				if (u>='A' && u<='F') u-='A'-10;
				else
				if (u>='a' && u<='f') u-='a'-10;
				rc[j++]=16*d+u;
			}
			break;
		default : rc[j++]=t[i++];
		}
	}	
	rc[j-1]=0;
	return rc;
}
