/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  ctrlfld.cpp
 *
 *  implements a class to manage MARC control fields
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h>
#include <ctype.h>
#include <stdlib.h>
                 
#include "ctrlfld.h"
#include "error.h"
                   
///////////////////////////////////////////////////////////////////////////////
//
// TCtrlSubfield
//
///////////////////////////////////////////////////////////////////////////////       
TCtrlSubfield::TCtrlSubfield(void)
{  
 itsSub				= '?'; 
 itsSubMandatory	= FALSE;
 itsSubRepeatable	= FALSE;
 itsSubOccurency	= 0; 
 itsNextSub			= NULL;
}

///////////////////////////////////////////////////////////////////////////////
//
// ~TCtrlSubfield
//
///////////////////////////////////////////////////////////////////////////////       
TCtrlSubfield::~TCtrlSubfield(void)
{	
	
} 
   
///////////////////////////////////////////////////////////////////////////////
//
// TControlField
//
///////////////////////////////////////////////////////////////////////////////       
TControlField::TControlField(void)
{  
 *itsTag			= 0; 
 itsTagMandatory	= FALSE;
 itsTagRepeatable	= FALSE;
 itsTagOccurency	= 0; 
 itsFirstIndicators	= NULL;
 itsSecondIndicators= NULL;
 itsFirstSubfield	= NULL; 
 itsLastSubfield	= NULL; 
 itsNextTag			= NULL;
}

///////////////////////////////////////////////////////////////////////////////
//
// ~TControlField
//
///////////////////////////////////////////////////////////////////////////////       
TControlField::~TControlField(void)
{ 
 if (itsFirstIndicators)	{ free(itsFirstIndicators);		itsFirstIndicators	= NULL;	}
 if (itsSecondIndicators)	{ free(itsSecondIndicators);	itsSecondIndicators	= NULL;	}
 DelTreeCheckSub(); 
}
   
///////////////////////////////////////////////////////////////////////////////
//
// SetFirstIndicators
//
///////////////////////////////////////////////////////////////////////////////   
int TControlField::SetFirstIndicators(char *Indicators)
{ 
 if (itsFirstIndicators)
 { 
	 free(itsFirstIndicators);
	 itsFirstIndicators	= NULL; 
 }
 if (*Indicators)
  if ((itsFirstIndicators=strdup(Indicators))==NULL)
   return ErrorHandler->SetError(2503,ERROR);
 return 0; 
} 

///////////////////////////////////////////////////////////////////////////////
//
// SetSecondIndicators
//
///////////////////////////////////////////////////////////////////////////////   
int TControlField::SetSecondIndicators(char *Indicators)
{ 
 if (itsSecondIndicators)	
 { 
	 free(itsSecondIndicators);	
	 itsSecondIndicators	= NULL; 
 }
 if (*Indicators)
  if ((itsSecondIndicators=strdup(Indicators))==NULL)
   return ErrorHandler->SetError(2504,ERROR);
 return 0; 
}  

///////////////////////////////////////////////////////////////////////////////
//
// DelTreeCheckSub
//
///////////////////////////////////////////////////////////////////////////////   
void TControlField::DelTreeCheckSub(void)
{ 
 TCtrlSubfield	*Courant;
 TCtrlSubfield	*Suivant;
 
 Courant = itsFirstSubfield;
 while (Courant)
  {
   Suivant = Courant->GetNextSub();
   delete Courant;
   Courant = Suivant;
  }
 itsFirstSubfield = itsLastSubfield = NULL; 
} 

///////////////////////////////////////////////////////////////////////////////
//
// TControlField
//
///////////////////////////////////////////////////////////////////////////////       
void TControlField::ResetSubOccurency(TCtrlSubfield *StartSub)
{
 TCtrlSubfield	*Current;
 
 Current=StartSub;
 while (Current)
  {
   Current->SetSubOccurency(0);
   Current=Current->GetNextSub();
  } 
}




