/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  error.cpp
 *
 *  implements a class to manage errors
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */
#include <afx.h>
#include "error.h"
#include "xvtstuff.h"
           
///////////////////////////////////////////////////////////////////////////////
//
// TError
//         
////////////////////////////////////////////////////////////////////////////////
          
TError::TError(TUMApplication* theApplication,char *LogFileName,char *ErrorCodeFileName)
{
 itsApplication=theApplication;
 
 itsTooManyErrors       = 0;
 itsHowManyErrors       = 0;
 itsErrorCode           = 0;
 itsDebugMode           = 0;
 itsLogError            = NULL;
 
 // Logfile opening
 OpenErrorLogFile(LogFileName);
 
 // Read the Error code file
 itsNbErrorCode=ReadErrorCodeFile(ErrorCodeFileName);
 
 // Temporary buffer allocation
 Temporary      = (unsigned char *)malloc(TBUF);
 Temporary2     = (unsigned char *)malloc(TBUF);
 Item           = (char *)malloc(BUFF_SIZE+1);
 Temporary[0] = '\0';
 Temporary2[0] = '\0';
 Item[0] = '\0';

}
                  
///////////////////////////////////////////////////////////////////////////////
//
// ~TError
//
////////////////////////////////////////////////////////////////////////////////
  
TError::~TError()
{    
	int            iIndice;
	
	for (iIndice=0; iIndice<itsNbErrorCode; iIndice++)
		free(itsErrorString[iIndice]); 
	
	itsNbErrorCode = 0;
	if (itsLogError)  
	{ 
		fclose(itsLogError);
		itsLogError = NULL; 
	}
	if (Temporary)    
	{ 
		free(Temporary);
		Temporary = NULL;
	}
	if (Temporary2)  
	{
		free(Temporary2);
		Temporary2 = NULL;
	}
	if (Item)                      
	{ 
		free(Item); 
		Item = NULL; 
	} 
}
 
///////////////////////////////////////////////////////////////////////////////
//
// SetErrorCode
//
////////////////////////////////////////////////////////////////////////////////
 
int TError::SetErrorCode(int ErrorCode,short Severity,char *FileName,int
						 LineNumber,char *UserData)
{
	short          iIndice;
	BOOLEAN        bFoundCode=FALSE;
	char           categorie[10];
	char           Message[1000];
	
	if (itsLogError)
	{                  
		for (iIndice=0; bFoundCode!=TRUE && iIndice<itsNbErrorCode; iIndice++)
		{
			if (itsErrorNumber[iIndice]==ErrorCode)
			{
				bFoundCode=TRUE;
			}
		}
		
		// Write message to logfile ...                 
		switch(Severity)
		{ 
		case FORCER        :
			sprintf(Message,"WARNING(%d)-%s\n%s",ErrorCode,bFoundCode==TRUE ? itsErrorString[iIndice-1] : "Unknown Error",UserData);
			Message[199]=0;             
			switch(xvt_dm_post_ask("Correct","Force", Message))
			{
			case RESP_2:
				return 0;
				break;                               
			default:
				return 0-ErrorCode;
				break;  
			}
			break;     
			case DISPLAY :
				sprintf(Message,"WARNING(%d)-%s\n%s",ErrorCode,bFoundCode==TRUE ? itsErrorString[iIndice-1] : "Unknown Error",UserData);
				Message[199]=0;
				xvt_dm_post_warning(Message);
				return 0-ErrorCode;
				break;         
			case WARNING : strcpy(categorie,"WARNING"); break;
			case ERROR   : strcpy(categorie," ERROR "); break;       
			default      : strcpy(categorie," FATAL "); break;          
		}
		
		// Error Code setting
		itsErrorCode=ErrorCode;
		itsHowManyErrors++;
		
		if ((ErrorCode>=9000) || (itsDebugMode)) 
			fprintf(itsLogError,"\n%s(%d)-%s : %s in %s:%d",categorie,ErrorCode, 
			bFoundCode==TRUE ? itsErrorString[iIndice-1] : "Unknown Error",UserData,FileName,LineNumber);
		else
			fprintf(itsLogError,"\n%s(%d)-%s : %s",categorie,ErrorCode,
			bFoundCode==TRUE ? itsErrorString[iIndice-1] : "Unknown Error",UserData);          
		fflush(itsLogError);        
		
		// Write message to screen
		if ((ErrorCode>=9000) || (itsDebugMode))  
			sprintf(Message,"%s(%d)-%s\n%s\nin %s:%d",categorie,ErrorCode,
			bFoundCode==TRUE ? itsErrorString[iIndice-1] : "Unknown Error",UserData,FileName,LineNumber);
		else
			sprintf(Message,"%s(%d)-%s\n%s",categorie,ErrorCode,
			bFoundCode==TRUE ? itsErrorString[iIndice-1] : "Unknown Error",UserData);           
		Message[199]=0;        
		switch(Severity)
		{
		case WARNING :
			if ((itsMode==INTERACTIVE) && (itsVerboseMode)) 
				xvt_dm_post_note(Message);
			break;
		case ERROR :        
			if ((itsMode==INTERACTIVE) && (itsVerboseMode))          
				switch(xvt_dm_post_ask("Continue","Stop", Message))
			{
		case RESP_2:
			break;                               
		default:
			break;  
			}
			break;
		default    :
			xvt_dm_post_note(Message);
			break;  
		}
	}  
	else
	{   
		sprintf(Message,"FATAL(%d)-%s",0,"Unable to write into the error log file\nPlease, check path and restart\n( see 'Initialization parameters' from the 'Options' Menu )");
		Message[199]=0;
		xvt_dm_post_note(Message);
	}  
	return 0-ErrorCode; 
}                  

///////////////////////////////////////////////////////////////////////////////
//
// GetErrorCode
//
////////////////////////////////////////////////////////////////////////////////

int TError::GetErrorCode(void)
{
 return itsErrorCode;
}
   
///////////////////////////////////////////////////////////////////////////////
//
// SetMode
//
////////////////////////////////////////////////////////////////////////////////
                                    
void TError::SetMode(int Mode)
{
 itsMode=Mode;
}

///////////////////////////////////////////////////////////////////////////////
//
// GetMode
//
////////////////////////////////////////////////////////////////////////////////
                                    
int TError::GetMode(void)
{
 return itsMode;
}
  
///////////////////////////////////////////////////////////////////////////////
//
// GetMode
//
////////////////////////////////////////////////////////////////////////////////
 
int TError::ReadErrorCodeFile(char *ErrorCodeFileName)
{
 FILE           *fpIn;
 char           ligne[201];
 char           tmp[1000];
 short          iLongueur;
 short          iIndice;
 int            iQuantite=0;
  
 if ((fpIn=fopen(ErrorCodeFileName,"r"))==NULL)
  return 0;

 while (fgets(ligne,150,fpIn))
  {
   iLongueur=(short)strlen(ligne);
   ligne[--iLongueur]=0;
   for (iIndice=0; iIndice<iLongueur; iIndice++)
    if (ligne[iIndice]=='|')
     break;
    else
     tmp[iIndice]=ligne[iIndice];
   tmp[iIndice]=0;
   itsErrorNumber[iQuantite]=atoi(tmp);
   iIndice++;
   while (iIndice<iLongueur && ligne[iIndice]==' ')
    iIndice++;
   itsErrorString[iQuantite]=strdup(&ligne[iIndice]);

   iQuantite++;
  }
 fclose(fpIn);
 return iQuantite;
}

///////////////////////////////////////////////////////////////////////////////
//
// OpenErrorLogFile
//
////////////////////////////////////////////////////////////////////////////////
 
int TError::OpenErrorLogFile(char *Name)
{
 time_t ns=time(NULL);

 if ((itsLogError=fopen(Name,"a"))==NULL)
  return SetErrorD(0,FATAL,Name);
 strcpy(itsErrorLogFileName,Name); 

 // John Hough - put in delete of asctime
 char *pcTime = asctime(localtime(&ns)); 
 if (pcTime)
 {
	fprintf(itsLogError,"\n ----------------------------------- \n Started at : %s\n\n",pcTime);
	free(pcTime);
 }
  fflush(itsLogError);
 return 0;
}



void TError::WriteError(char *message)
{
	if (itsLogError)
	{
		fprintf(itsLogError,"%s",message);
        fflush(itsLogError);
	}
}
