/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  marcfile.cpp
 *
 *  implements a class to manage input and output of MARC records
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h> 
#include "marcfile.h"
#include "error.h"
   
// John Hough - added extern g_itsDocument;
extern TMarcDoc *g_itsDocument;

// John Hough
#include "objectlist.h"

 
///////////////////////////////////////////////////////////////////////////////
//
// TMarcFile
//
///////////////////////////////////////////////////////////////////////////////   
TMarcFile::TMarcFile( FILE_SPEC *FileSpec, char Mode, char Kind)
		  :TFile(FileSpec, Mode, Kind)
{

 itsDocument= new TMarcDoc;

 // John Hough - added global reference
 g_itsDocument = itsDocument;


 Buffer		= NULL;  
 Buf		= new unsigned char[TBLMAX];
 SetMarcInfoFormat(itsDocument->GetMarcFileFormat());
 SetMarcInfoBlockSize(itsDocument->GetMarcFileBlockSize());
 SetMarcInfoMinDataFree(itsDocument->GetMarcFileMinDataFree());
 SetMarcInfoPaddingChar(itsDocument->GetMarcFilePaddingChar());
 SetMarcInfoLastBlock(itsDocument->GetMarcFileLastBlock());
}
   
///////////////////////////////////////////////////////////////////////////////
//
// ~TMarcFile
//
///////////////////////////////////////////////////////////////////////////////   
TMarcFile::~TMarcFile() 
{ 
 if (iFile!=-1)		{ Close();			iFile 	= -1;	}
 if (Buffer)		{ free(Buffer);		Buffer	= NULL;	}
 if (Buf)			{ delete Buf;		Buf		= NULL;	}
  

}  
 
///////////////////////////////////////////////////////////////////////////////
//
// Open
//
///////////////////////////////////////////////////////////////////////////////   
int TMarcFile::Open() 
{
 if (TFile::Open())			// Echec d'ouverture du fichier
  return 1;
 
 NumNotice		= 0L;
 PosCour		= 0L;
 NumBloc		= 0L;
 TB				= GetMarcInfoBlockSize();
 if (!Buffer)
  if ((Buffer=(unsigned char *)malloc(TBUF))==NULL)
   return ErrorHandler->SetError(1501,ERROR);
 EndOfFile		= 0;
 Eof			= 0;
 PBuf			= 0;
 TBuf			= 0;
 *Buf			= 0;
 memset(FinBande,GetMarcInfoPaddingChar(),5);
 return 0;    
}

///////////////////////////////////////////////////////////////////////////////
//
// Reset
//
///////////////////////////////////////////////////////////////////////////////
int TMarcFile::Reset(void)
{
 if (itsMode==FILE_WRITE)
  return ErrorHandler->SetError(1001,ERROR);

 lseek(iFile,0L,0);
 NumNotice	= 0L;
 PosCour	= 0L;
 NumBloc	= 0L;
 TB			= GetMarcInfoBlockSize();
 EndOfFile	= 0;
 Eof		= 0;
 PBuf		= 0;
 TBuf		= 0;
 *Buf		= 0;
 return 0;
}
 
///////////////////////////////////////////////////////////////////////////////
//
// val
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::val(unsigned char* buffer, unsigned short *valeur)
{
 unsigned int	i;

 *valeur	= 0;
 for (i=0; i<strlen((const char *)buffer); ++i)
  if (!isdigit(buffer[i]))
   return 1;
 *valeur	= (unsigned short)atoi((const char *)buffer);
 return 0;
}
     
///////////////////////////////////////////////////////////////////////////////
//
// longval
//
///////////////////////////////////////////////////////////////////////////////      
int TMarcFile::longval(unsigned char* buffer, unsigned long *valeur)
{
 unsigned int	i;

 *valeur=0;
 for (i=0; i<strlen((const char *)buffer); ++i)
  if (!isdigit(buffer[i]))
   return 1;
 *valeur = atol((const char *)buffer);
 return 0;
}
  
///////////////////////////////////////////////////////////////////////////////
//
// debut_bloc
//
///////////////////////////////////////////////////////////////////////////////   
int TMarcFile::debut_bloc()
{
 if (PosCour%GetMarcInfoBlockSize())
  return 0;
 else
  return 1;
}

///////////////////////////////////////////////////////////////////////////////
//
// lire_scw
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::lire_scw(char debut)
{
 unsigned char	scw[6];
 unsigned short	vscw;
                  
 if (GetMarcInfoFormat()!=SEGMENTED)
  return 0;

 if (lire(5,scw))     				// Lecture du SCW
  if (!debut)
   return ErrorHandler->SetError(1003,ERROR);
  else
   return 1;

 PosCour += 5;
 NumBloc = PosCour/GetMarcInfoBlockSize();
 FinBloc = (NumBloc+1)*GetMarcInfoBlockSize();

 if (!memcmp(scw,FinBande,5  )) // fin de la bande
  {
   EndOfFile	= 1;
   return 1;
  }
     
 if (val(scw,&vscw))    			// Verification de sa valeur
  return ErrorHandler->SetErrorD(1002,ERROR,(char *)scw);

 return 0;
}


///////////////////////////////////////////////////////////////////////////////
//
// lire
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::lire(unsigned short taille, unsigned char * buffer)
{
 unsigned short	pbuf,
 				lreste;
 short			lng;

 pbuf=0;
 lreste=taille;
 while (lreste>0)
  {
   Reste=(unsigned short)(TBuf-PBuf);
   if (Reste>=lreste)
    {
     memcpy(&buffer[pbuf],&Buf[PBuf],lreste);
     buffer[pbuf+lreste]=0;
     PBuf = (unsigned short)(PBuf + lreste);
     lreste=0;
    }
   else
    {
     if (Eof)
      {
       EndOfFile=1;
       return 1;
	 } 
     memcpy(&buffer[pbuf],&Buf[PBuf],Reste);
     pbuf = (unsigned short)(pbuf + Reste);
     lreste = (unsigned short)(lreste - Reste);
     

	 // John Hough
	 if (myrun.GetReadMarcRecordsFromBexFile())
		lng = (short)::read(iFile,Buf,TB);
	 else
	 {
		 // John Hough
		// MARC IN
		lng = 1537;
		char *pcRecord;
		int iLength = 0;
		myrun.GetInputMarcRecord(pcRecord, iLength);
		memcpy(Buf, pcRecord, iLength);
	 }

     if (lng==-1)
      return ErrorHandler->SetError(1502,ERROR);
     else
      if (lng<TB)
       Eof=1;
     TBuf=lng;
     PBuf=0;
    }
  }
 return 0;
}







///////////////////////////////////////////////////////////////////////////////
//
// Read
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::Read(TUMRecord *Record)
{
 unsigned long	lalire,
 				lbuf,
 				lreste;
 unsigned char	temp[TBLMAX];

 FinBloc=GetMarcInfoBlockSize()*(NumBloc+1);

 if ((FinBloc-PosCour<=(unsigned long)(GetMarcInfoMinDataFree())) &&
     (GetMarcInfoFormat()==SEGMENTED))
  {
   // S'il ne reste pas suffisament de caracteres disponibles dans le bloc, il est
   // impossible d'y lire un SCW, on saute donc au bloc suivant
   if (lire((unsigned short)(FinBloc-PosCour),temp))
    if (EndOfFile)
     return 1;
    else
     return ErrorHandler->SetError(1003,ERROR);

   ++NumBloc;
   PosCour=NumBloc*GetMarcInfoBlockSize();
   FinBloc=PosCour+GetMarcInfoBlockSize();
  }

 if (lire_scw(1))
  return 1;

 if (FinBloc-PosCour < (unsigned long)(GetMarcInfoMinDataFree()))
  // On essai maintenant de lire les 5 premiers caracteres du guide, pour  avoir la
  // longueur de la notice. Mais ces 5 caracteres peuvent-etre a cheval sur deux blocs,
  // s'il reste moins de 5 caracteres dans le bloc. Un SCW sera donc present. Ce que 
  // l'on traite ici 
  {
   lalire = FinBloc-PosCour;
   lreste = 5-lalire;
   if (lire((short)lalire,Buffer))
    return ErrorHandler->SetError(1003,ERROR);

   ++NumBloc;
   PosCour=NumBloc*GetMarcInfoBlockSize();
   FinBloc=PosCour+GetMarcInfoBlockSize();

   if (lire_scw(0)) 
    return 1;

   // On se trouve en debut de bloc, il faut donc lire le SCW. Si on y arrive pas, il
   // y a forcement erreur, puisqu'on se trouve dans les 5 premiers caracteres du guide

   if (lire((unsigned short)lreste,&Buffer[lalire]))
    return ErrorHandler->SetError(1003,ERROR);

   PosCour+=lreste;
   // Buffer contient maintenant les 5 premiers caracteres du guide
  }
 else
  // Les 5 premiers caracteres du guide, ici, sont tous dans le meme bloc ... on essai
  // donc de les lire
  {
   if (lire(5,Buffer))
    return ErrorHandler->SetError(1003,ERROR);

   PosCour+=5;
  }

 if (longval(Buffer,&LngNotice))
 // On verifie que la longueur est numerique, et on affecte sa valeur a E_LNG_NOTICE
  return ErrorHandler->SetErrorD(1004,ERROR,(char *)Buffer);

 lbuf = 5;
 lreste = LngNotice-5;
 do
  // On lit maintenant le reste de la notice, que l'on concatene au BUFFER contenant
  // les 5 premiers caracteres du guide, pour avoir dans BUFFER la notice complete
  {
   if (debut_bloc())
    // Si on est en debut de bloc, il faut passer par dessus le SCW de debut de bloc
    if (lire_scw(0))
     return 1;

   lalire = lreste;
   // On lit une notice morceau par morceau, chaque morceau s'arretant soit a la fin
   // de la notice, soit a la fin du bloc courant.
   //     - lbuf est la longueur deja stockee dans Buffer
   //     - lreste est la longueur qui reste a lire pour avoir toute la notice
   //     - lalire est la longueur a lire dans le bloc courant

   if (lalire+PosCour>FinBloc)
    lalire=FinBloc-PosCour;

   if (lire((short)lalire,&Buffer[lbuf]))
    return ErrorHandler->SetError(1003,ERROR);

   lbuf += lalire;
   lreste -= lalire;
   PosCour += lalire;
   NumBloc=PosCour/GetMarcInfoBlockSize();
   FinBloc=(NumBloc+1)*GetMarcInfoBlockSize();
  }
 while (lreste>0);
             
 // La notice contenue dans le buffer est dispatche en enregistrement Marc
 Record->FromString((char *)(Buffer));
 ++NumNotice;
 
 return 0;
}



///////////////////////////////////////////////////////////////////////////////
//
// Write
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::Write(TUMRecord *Record)
{
 unsigned long	pos,
 				laecrire,
 				lreste;
 unsigned char	temp[TBLMAX];

 // La notice contenue dans un enregistrement marc est regroupe dans une chaine
 Record->ToString((char *)(Buffer),TBUF);
 
 // John Hough
 if (!myrun.GetReadMarcRecordsFromBexFile())
	myrun.SetMarcData((char *)Buffer); // a SetMarcLength 


 lreste=LngNotice=strlen((const char *)Buffer);
 if (lreste==0)
  return 0;

 ++NumNotice;
 FinBloc=(NumBloc+1)*GetMarcInfoBlockSize();

 if ((FinBloc-PosCour<=(unsigned long)(GetMarcInfoMinDataFree())) &&
     (GetMarcInfoFormat()==SEGMENTED))
 // S'il ne reste pas assez de caracteres dans le bloc, on passe au bloc suivant
  {
   memset(temp,GetMarcInfoPaddingChar(),GetMarcInfoMinDataFree());
   temp[GetMarcInfoMinDataFree()]=0;
   if (ecrire((unsigned short)(FinBloc-PosCour),temp))
    return ErrorHandler->SetError(1005,ERROR);

   ++NumBloc;
   PosCour=NumBloc*GetMarcInfoBlockSize();
   FinBloc=PosCour+GetMarcInfoBlockSize();
  }

 pos=0;
 do
  {
   laecrire=lreste;
   if (GetMarcInfoFormat()==SEGMENTED)
    {
     if ( laecrire+PosCour+5 > FinBloc )
     // La notice ne finira pas dans ce bloc
      {
       laecrire=FinBloc-PosCour-5;
       if (pos==0)
       // La notice commence dans ce bloc   --> SCW =1....
        {
         if (ecrire_scw(1,(short)laecrire))
          return 1;
        }
       // La notice ne commence pas dans ce bloc --> SCW =2.... 
       else
        if (ecrire_scw(2,(short)laecrire))
         return 1;
      }
     else
     // La notice finira dans ce bloc
      {
       if (pos==0)
       // La notice commence dans ce bloc   --> SCW =0....
        {
         if (ecrire_scw(0,(short)laecrire))
          return 1;
        }
       // La notice ne commence pas dans ce bloc --> SCW =3....
       else
        if (ecrire_scw(3,(short)laecrire))
         return 1;
      }
    }

   if (ecrire( (short)laecrire, &Buffer[pos] ))
    return ErrorHandler->SetError(1005,ERROR);

   // On ecrit ce bout de notice dans le bloc
   PosCour+= laecrire;
   NumBloc=PosCour/(GetMarcInfoBlockSize());
   FinBloc=(NumBloc+1)*(GetMarcInfoBlockSize());
   pos += laecrire;
   lreste -= laecrire;
  }
 while (lreste>0);  // On a fini quand toute la notice est ecrite 
 return 0;
}




///////////////////////////////////////////////////////////////////////////////
//
// ecrire_scw
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::ecrire_scw(short typ,unsigned short longueur)
{
 unsigned char	scw[6];

 if (GetMarcInfoFormat()!=SEGMENTED)
  return 0;

 sprintf((char *)scw,"%1d%04d",typ,longueur+5);
 scw[5]=0;
 if (ecrire(5,scw))
  return ErrorHandler->SetError(1005,ERROR);

 PosCour += 5;
 NumBloc=PosCour/GetMarcInfoBlockSize();
 FinBloc=(NumBloc+1)*GetMarcInfoBlockSize();
 return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// ecrire
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::ecrire( unsigned short taille, unsigned char* buffer )
{
 unsigned short	pbuf,
 				lreste;

 lreste=taille;
 pbuf=0;
 while (lreste>0)
  {
   Reste=(unsigned short)(TB-PBuf);
   if (Reste>=lreste)
    {
     memcpy(&Buf[PBuf],&buffer[pbuf],lreste);
     PBuf =(unsigned short)(PBuf + lreste);
     lreste=0;
    }
   else
    {
     memcpy(&Buf[PBuf],&buffer[pbuf],Reste);
     pbuf = (unsigned short)(pbuf + Reste);
     lreste = (unsigned short)(lreste - Reste);
     if (write(iFile,Buf,TB)!=TB)
      return 1;
     PBuf=0;
    }
  }
 return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// Close
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcFile::Close()
{
 int		i;
 
 if (itsMode==FILE_WRITE)
  {
   if (GetMarcInfoLastBlock())
    {
     for (i=PBuf;i<TB;i++)
      Buf[i] = GetMarcInfoPaddingChar();
     Buf[TB] = 0;
     PBuf=(unsigned short)TB;
    }
   if ((unsigned long)write(iFile,Buf,PBuf)!=PBuf)
    return ErrorHandler->SetError(1006,ERROR);

   *Buf=0;
   PBuf=0;
  }

 TFile::Close();
 return 0;
}


























