/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  mrcfiled.cpp
 *
 *  implements a class to manage MARC record fields
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h>
#include <ctype.h>
#include <stdlib.h>
   
#include "mrcfield.h"
#include "error.h"
         
#include "dtcnfdlg.h"
                   
                   
///////////////////////////////////////////////////////////////////////////////
//
// TMarcField
//
///////////////////////////////////////////////////////////////////////////////       
TMarcField::TMarcField(void)
{  
 *itsTag			= 0;
 itsIndicators[0]	= 0;
 itsIndicators[1]	= 0; 
 itsIndicators[2]	= 0;
 itsLib				= NULL;
 itsNextField		= NULL;
}

///////////////////////////////////////////////////////////////////////////////
//
// TMarcField : copie constructeur recursif
//
///////////////////////////////////////////////////////////////////////////////       
TMarcField::TMarcField(TMarcField *aField)
{  
 strcpy(itsTag,aField->GetTag());
 itsIndicators[0]=aField->GetI1();
 itsIndicators[1]=aField->GetI2(); 
 itsIndicators[2]=0;
 SetLib(aField->GetLib());
 if (aField->GetNextField())
  itsNextField=new TMarcField(aField->GetNextField());
 else
  itsNextField=NULL;
}
       
///////////////////////////////////////////////////////////////////////////////
//
// ~TMarcField
//
///////////////////////////////////////////////////////////////////////////////       
TMarcField::~TMarcField(void)
{ 
 if (itsLib)	{ free(itsLib);	itsLib	= NULL;	};
}
     

  
///////////////////////////////////////////////////////////////////////////////
//
// NextSubField
//
/////////////////////////////////////////////////////////////////////////////// 
int TMarcField::NextSubField(int *_Position,unsigned char* _SubField,unsigned char* _Result)
{ 
 short		iIndice=0;
 
 if (itsLib[*_Position]==0)
  return 0;						
  
 if (*_SubField!='*')
  {
   while (((itsLib[*_Position]!=31) || (itsLib[(*_Position)+1]!=*_SubField)) && (itsLib[(*_Position)+1]!=0))
    (*_Position)++; 
   if (!itsLib[(*_Position)+1])
    return 0;    
   if (itsLib[(*_Position)+1]!=*_SubField)
    return 0;
   else
    {
     if (itsLib[(*_Position)+2])
      (*_Position)+=2;
     else
      return 0; 
     while ((itsLib[*_Position]!=31) && (itsLib[*_Position]!=0))
      _Result[iIndice++]=itsLib[(*_Position)++];
     _Result[iIndice]=0;
     return 1;
    }
  }
 else
  {
   while ((itsLib[*_Position]!=31) && (itsLib[*_Position+1]!=0))
    (*_Position)++;
   if (itsLib[*_Position]!=31)
    return 0;
   else
    { 
     if (!itsLib[(*_Position)+1])
      return 0;
     *_SubField=itsLib[*_Position+1];
     if (itsLib[(*_Position)+2])
      (*_Position)+=2;
     else
      return 0;      
     while ((itsLib[*_Position]!=31) && (itsLib[*_Position]!=0))
      _Result[iIndice++]=itsLib[(*_Position)++];
     _Result[iIndice]=0;
     return 1;
    }
  }
}           
  
///////////////////////////////////////////////////////////////////////////////
//
// SetTag
//
///////////////////////////////////////////////////////////////////////////////   
int TMarcField::SetTag(char *aTagString)
{ 
 int		CurrentPos;
 
 if (strlen(aTagString)>3)
  return 1;

 for (CurrentPos=0; CurrentPos<3; CurrentPos++)
  if (!isalnum(aTagString[CurrentPos]))
   return 1;
   
 strcpy(itsTag,aTagString);
 return 0;
} 

int TMarcField::SetTag(int aTagNumber)
{    
 if ((aTagNumber<1 ) || (aTagNumber>999))
  return 1;
  
 sprintf(itsTag,"%03d",aTagNumber);
 return 0;
};

///////////////////////////////////////////////////////////////////////////////
//
// SetIndicators
//
///////////////////////////////////////////////////////////////////////////////   
int	TMarcField::SetIndicators(char *theIndicators)
{
 if ((strlen(theIndicators)!=2) && (strlen(theIndicators)!=0))
  return 1;
 strcpy(itsIndicators,theIndicators);
 return 0; 
}
 
///////////////////////////////////////////////////////////////////////////////
//
// SetLib
//
///////////////////////////////////////////////////////////////////////////////   
int TMarcField::SetLib(char *aLib)
{
 if ((!aLib) || (!(*aLib)))
  return 0;
   
 itsLib = (char *)malloc(strlen(aLib)+1);
 if (itsLib==NULL)
  return 1;
 strcpy(itsLib,aLib); 
 return 0; 
}
int TMarcField::SetLib(char *aLib,unsigned int aSize)
{
 itsLib	= (char *)malloc(aSize+1); 
 if (!itsLib)
  return 1;
 memcpy(itsLib,aLib,aSize);
 itsLib[aSize]=0;
 return 0;
}
 
 
 
 
