/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  rulefile.cpp
 *
 *  implements a class to manage the rule files
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */
 
#include <afx.h>
#include "rulefile.h"
#include "error.h"
#include "tools.h"     

// Fonction d'evaluation par lex & yacc
int Init_Evaluate_Rule(void *, int);
int Evaluate_Rule( TUMRecord*, TUMRecord*, TRule* );
int End_Evaluate_Rule();

extern TUMApplication *g_theApplication; 

///////////////////////////////////////////////////////////////////////////////
//
// TRuleFile
//
///////////////////////////////////////////////////////////////////////////////   
TRuleFile::TRuleFile(FILE_SPEC *FileSpec)
		  :TFile( FileSpec)
{
 itsFirstRule 		= NULL;
 itsLastInputCD		= NULL;
 itsLastOutputCD	= NULL;
 itsFirstCodedData	= NULL;
 itsLastCodedData	= NULL;
}
   
///////////////////////////////////////////////////////////////////////////////
//
// ~TRuleFile
//
///////////////////////////////////////////////////////////////////////////////   
TRuleFile::~TRuleFile() 
{
 // John Hough - added test an delete
 if (itsDocument)
	 delete itsDocument;


 // Unload the Rule Tree from memory
 DelTreeRule();
 DelTreeCodedData();
 if (itsLastInputCD)	{ delete itsLastInputCD;	itsLastInputCD	= NULL; }
 if (itsLastOutputCD)	{ delete itsLastOutputCD;	itsLastOutputCD	= NULL; }
 



}  
 
///////////////////////////////////////////////////////////////////////////////
//
// Close
//
///////////////////////////////////////////////////////////////////////////////   
int TRuleFile::CloseRuleFile() 
{  
 // Unload the Rule tree from memory
 DelTreeRule();
 DelTreeCodedData();
 if (itsLastInputCD)	{ delete itsLastInputCD;	itsLastInputCD	= NULL; }
 if (itsLastOutputCD)	{ delete itsLastOutputCD;	itsLastOutputCD	= NULL; }
 return 0;
} 
 
///////////////////////////////////////////////////////////////////////////////
//
// Open
//
///////////////////////////////////////////////////////////////////////////////   
#define RULE_SIZE	BUFF_SIZE *4
int TRuleFile::OpenRuleFile() 
{                    
 char		RuleLine[RULE_SIZE+1];
 char		CommentLine[RULE_SIZE+1];
 char		Illustration[(2*RULE_SIZE)+1];
 char*		PreviousComment=NULL;
 TRule		*CurrentRule;
 FILE_SPEC	IncludedFileSpec;
 int		Result;
 int		Line;
 BOOLEAN	IsRuleAnalysed;
 
 // Verify that the Rule File exists
 if (Exists()==FALSE)
  return ErrorHandler->SetErrorD(5001,WARNING,itsFileInfo->name);
      
 // Open the Rule file in Reading mode 
 itsMode = FILE_READ;    
 if (Open())
  return 1;

 itsDocument = new TRuleDoc();

// ((TRuleDoc *)itsDocument)->SetInputFormat(itsFirstLine);
// ((TRuleDoc *)itsDocument)->SetOutputFormat(itsSecondLine);

	g_theApplication->itsRuleDoc->SetInputFormat(itsFirstLine);
	g_theApplication->itsRuleDoc->SetOutputFormat(itsSecondLine);
 
 // Load the rule on into a Rule memory tree
 CurrentRule = new TRule;
 if (!CurrentRule)
  return ErrorHandler->SetError(5501,ERROR);
 CurrentRule->SetPreviousRule(NULL);
 itsFirstRule = CurrentRule;
 IsRuleAnalysed =FALSE;
 while (!NextLine(RuleLine,RULE_SIZE,CommentLine,RULE_SIZE,&IncludedFileSpec,&Line))	// Read a Line from the Rule File
  {
   sprintf(Illustration,"in file '%s' at line %d :\n%s",IncludedFileSpec.name,Line-1,RuleLine);   
   if ((strchr(RuleLine,'|')) && (IsRuleAnalysed==TRUE))
   // this is a new rule to proceed
    {
     if (itsLastInputCD)	{ delete itsLastInputCD;	itsLastInputCD	= NULL; }
     if (CurrentRule->GetInputCD())
      itsLastInputCD = new TCD(CurrentRule->GetInputCD());
     if (itsLastOutputCD)	{ delete itsLastOutputCD;	itsLastOutputCD	= NULL; }
     if (CurrentRule->GetOutputCD())
      itsLastOutputCD = new TCD(CurrentRule->GetOutputCD());
     CurrentRule->SetNextRule(new TRule(itsLastInputCD,itsLastOutputCD));						// Memory loading passed
     if (!CurrentRule->GetNextRule())
      return  ErrorHandler->SetErrorD(5501,ERROR,Illustration);
     CurrentRule->GetNextRule()->SetPreviousRule(CurrentRule);
     CurrentRule = CurrentRule->GetNextRule();
    }
   else
    {
     if (*RuleLine==0 && *CommentLine)
      // Cette ligne ne possede qu'un commentaire
      {
       if (PreviousComment)
        {
         strcpy((char*)Temporary,PreviousComment);
         delete PreviousComment;
        }
       else *Temporary=0;
       PreviousComment=new char[strlen((char*)Temporary)+strlen(CommentLine)+2];
       strcpy(PreviousComment,(char*)Temporary);
       if (*Temporary)
        strcat(PreviousComment,"\n");
       strcat(PreviousComment,CommentLine);
       continue;
      }
    }

   if (PreviousComment)
    {
     strcpy((char*)Temporary,PreviousComment);
     delete PreviousComment;
     PreviousComment=NULL;
    }
   else 
    *Temporary=0;
   strcat((char*)Temporary,CommentLine);
 
   if ((Result=CurrentRule->FromString(RuleLine,(char*)Temporary))<0)	// Load Input and Output CDs in CDs
   // Error on rule
    {
     ErrorHandler->SetErrorD(-Result,ERROR,Illustration);
     // Delete the erronous rule
     if (CurrentRule->GetInputCD())	{ delete(CurrentRule->GetInputCD());	CurrentRule->SetInputCD(NULL);	}
     if (CurrentRule->GetOutputCD()){ delete(CurrentRule->GetOutputCD());	CurrentRule->SetOutputCD(NULL);	}
     if (CurrentRule->GetLib())		{ delete(CurrentRule->GetLib());		CurrentRule->SetLib(NULL);	}
     if (CurrentRule->GetComment())	{ delete(CurrentRule->GetComment());	CurrentRule->SetComment(NULL);	}
     IsRuleAnalysed = FALSE;
    }
   else
    if (Result>0)
     IsRuleAnalysed = TRUE; 
  }
 CurrentRule->SetNextRule(NULL); 
                                            
 // Close the Rule file             
 Close();
               
 return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// Save
//
///////////////////////////////////////////////////////////////////////////////   
int TRuleFile::SaveRuleFile() 
{     
 TRule		*CurrentRule;
 int		ReturnCode;
        
 // Open the Rule file in Writing mode 
 itsMode = FILE_WRITE;    
 if (Open())
  return 1;

 fprintf(File,"%s\n%s\n",((TRuleDoc *)itsDocument)->GetInputFormat(),((TRuleDoc *)itsDocument)->GetOutputFormat());
 
 CurrentRule = itsFirstRule;
 while (CurrentRule)
  {
   if ((ReturnCode=CurrentRule->ToString((char *)Temporary,(char *)Temporary2))==0)
   // Regle avec CD-IN et CD-OUT renseigns
    {
     if (*Temporary2)
     // Regle avec commentaire         
      fprintf(File,"\n%s",(char *)Temporary2);
     // Ecriture de la regle 
     fprintf(File,"\n%s",(char *)Temporary);
    }
   else
    ErrorHandler->SetErrorD(5200,ERROR,(char *)Temporary); 
   CurrentRule = CurrentRule->GetNextRule();
  }
 fflush(File);
  
 Close();

 return 0;
}
 
///////////////////////////////////////////////////////////////////////////////
//
// Convert
//
// Cette methode convertit un UMRecord sous forme de CDLib en un autre, 
// conformement au fichier de regles
//
///////////////////////////////////////////////////////////////////////////////   

// John Hough
#include "ytab.h"
extern YYSTYPE yylval;
#include "typedef.h"
extern "C" {
extern TypeInst *g_yylvalinstr;
};

int
TRuleFile::Convert( TUMRecord* In, TUMRecord* Out )
 {
  TRule*  aRule=itsFirstRule;
  //static int counter = 0; // John Hough
//	int a; // John Hough

  ErrorHandler->Reset();

  Init_Evaluate_Rule(itsDocument,g_theApplication->GetExecutionMode());

  // On parcourt chaque regle, et on en fait l'evaluation
  while( aRule )
   {
    if (*(aRule->GetOutputCD()->GetTag())!='?')
     {
      if (aRule->GetOutputCD()->GetIN())
       {
        Evaluate_Rule( In, In, aRule );
        In->SortCD();
       }
      else
	  {
		// John Hough tester - added counter abnd brackets
       //printf("counter = %d\n", counter);
	   //counter++;
	   //if (yylval.inst)
			//a = 2;
	   //if (g_yylvalinstr)
		   //a = 2;

		 Evaluate_Rule( In, Out, aRule );

		 //if (g_yylvalinstr)
		 //{
		   //a = 2;
		   //free(g_yylvalinstr);
		 //}
		 
	  }
     }
	

	//if (g_yylvalinstr)
	//{
	//	   a = 2;
	//}

    aRule=aRule->GetNextRule();
   }   

 End_Evaluate_Rule();

 return ErrorHandler->GetErrorCode();
}

///////////////////////////////////////////////////////////////////////////////
//
// DelTreeRule
//
///////////////////////////////////////////////////////////////////////////////    
void TRuleFile::DelTreeRule(void)
{
 TRule			*Courant,
 				*Suivant;
 Courant=itsFirstRule;

 while(Courant)
  {
   Suivant=Courant->GetNextRule();
   delete Courant;
   Courant=Suivant;
  }
 itsFirstRule=NULL;
}

///////////////////////////////////////////////////////////////////////////////
//
// DelTreeCodedData
//
///////////////////////////////////////////////////////////////////////////////    
void TRuleFile::DelTreeCodedData(void)
{
 TCodedData		*Courant,
 				*Suivant;
 Courant=itsFirstCodedData;

 while(Courant)
  {
   Suivant=Courant->GetNextCodedData();
   delete Courant;
   Courant=Suivant;
  }
 itsFirstCodedData=NULL;
}
 
///////////////////////////////////////////////////////////////////////////////
//
// GetCodedData
//
///////////////////////////////////////////////////////////////////////////////    
TCodedData	*TRuleFile::GetCodedData(char *theName)
{ 
 FILE_SPEC		CodedDataSpec;
 FILE_SPEC		*testspec;
 TCodedData		*aCodedData,
 				*anOldCodedData;
 char			UpperName[BUFF_SIZE+1]; 				
 
 ErrorHandler->Reset();
 testspec = ((TRuleDoc *)g_theApplication->itsRuleDoc)->GetRuleSpec();
 memcpy(&CodedDataSpec,testspec,sizeof(FILE_SPEC));
 strcpy(CodedDataSpec.name,theName); 
 strcpy(UpperName,CodedDataSpec.name);
 ToUpperCase(UpperName);
 if (!strchr(UpperName,'.'))
  {
   strcat(CodedDataSpec.name,".tbl");
   strcpy(CodedDataSpec.type,".tbl");
  } 

 anOldCodedData=itsFirstCodedData;
 while(anOldCodedData)
  { 
   if (!memcmp(anOldCodedData->GetCodedDataName(),&CodedDataSpec,sizeof(FILE_SPEC)))
   // the coded data table was found
    return anOldCodedData;
   anOldCodedData = anOldCodedData->GetNextCodedData(); 
  }
 // this coded data table doesn't exist 

 aCodedData=new TCodedData(&CodedDataSpec);
 anOldCodedData = itsLastCodedData;

 if (aCodedData)
  // Succed in TCodedData creation
  if (!aCodedData->Open())
   // Succed in coded data table loading from file
   {
    if (!anOldCodedData) 
    // there is no coded data table loaded ==> load this one to the head of the list
     itsFirstCodedData=itsLastCodedData=aCodedData;
    else
    // there is one table loaded at least ==> load this table to the end of the list
     {
      anOldCodedData->SetNextCodedData(aCodedData);
      itsLastCodedData=aCodedData;
     }
    return aCodedData;
   }              
   
 // coded data table loading from file or TcodedData creation failure
 delete aCodedData;
 return NULL; 
}




























