/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  tcdlib.cpp
 *
 *  implements a set of functions to manage control data fields
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h> 
#include "tcdlib.h"
#include "error.h" 

///////////////////////////////////////////////////////////////////////////////
//
// TCDLib
//
///////////////////////////////////////////////////////////////////////////////  
TCDLib::TCDLib(void):TCD()
{  
 itsContent	= NULL;
}
     
///////////////////////////////////////////////////////////////////////////////
//
// TCDLib : copie constructeur 
//
///////////////////////////////////////////////////////////////////////////////       
TCDLib::TCDLib(TCDLib *aCDLib, TCD*aCD)
{
 _IN=aCDLib->GetIN(); 
 strcpy(itsTag,aCDLib->GetTag());
 strcpy(itsSubfield,aCDLib->GetSubfield());
 itsOccurenceNumber=aCDLib->GetOccurenceNumber();
 itsTagOccurenceNumber=aCDLib->GetTagOccurenceNumber(); 
 itsSubOccurenceNumber=aCDLib->GetSubOccurenceNumber();
 SetBegining(aCDLib->GetBegining());
 SetEnd(aCDLib->GetEnd()); 
 if (aCDLib->GetNext())
  {
   itsNext = (TCD *)new TCDLib((TCDLib*)aCDLib->GetNext());
   itsNext->SetPrevious((TCD*)this);
  } 
 else
  itsNext = NULL;
 itsPrevious = NULL;  
 itsContent=NULL; 
 SetContent(aCDLib->GetContent(),aCD); 
}  

///////////////////////////////////////////////////////////////////////////////
//
// TCDLib : copie constructeur 
//
///////////////////////////////////////////////////////////////////////////////       
TCDLib::TCDLib(TCD *aCD)
{  
 _IN=aCD->GetIN();
 strcpy(itsTag,aCD->GetTag());
 strcpy(itsSubfield,aCD->GetSubfield());
 itsOccurenceNumber=aCD->GetOccurenceNumber();
 itsTagOccurenceNumber=aCD->GetTagOccurenceNumber(); 
 itsSubOccurenceNumber=aCD->GetSubOccurenceNumber();
 SetBegining(aCD->GetBegining());
 SetEnd(aCD->GetEnd()); 
 if (aCD->GetNext())
  {
   itsNext = (TCD *)new TCDLib(aCD->GetNext());
   itsNext->SetPrevious((TCD*)this);
  } 
 else
  itsNext = NULL;
 itsPrevious = NULL;  
 itsContent=NULL; 
}  
      
///////////////////////////////////////////////////////////////////////////////
//
// ~TCDLib
//
///////////////////////////////////////////////////////////////////////////////   
TCDLib::~TCDLib(void)
{
 if (itsContent)	{ free(itsContent);		itsContent	= NULL;	}
}  

///////////////////////////////////////////////////////////////////////////////
//
// GetContent
//
// Cette methode retourne le contenu du CDLib correspondant a la description faite
// dans aCD. Trois cas sont possibles :
// - ou bien aCD est NULL, on renvoi simplement le contenu complet du CDLib.
// - ou bien aCD decrit completement le CDLib (c'est-a-dire que le sous-champ est precise),
//   on fournit alors le contenu du CDLib correspondant a l'intervalle specifie dans aCD.
// - ou bien aCD decrit un champ (pas de subfield de defini), auquel cas on doit rendre
//   le contenu complet du champ, en concatenant les contenus de l'ensemble des CDLib
//   correspondant ... et en ne prenant en final que l'intervalle voulu.
//
///////////////////////////////////////////////////////////////////////////////   
char *TCDLib::GetContent(TCD* theCD)
{ 
 TCD* aCD;
 // Si aCD est NULL on retourne simplement le contenu complet du CDLib

 if (theCD==NULL) return itsContent;
 else
  {
   aCD=new TCD(theCD);
   aCD->SetTagOccurenceNumber(GetTagOccurenceNumber());
  }


 // Si le sous-champ n'est pas precise, il faut concatener tous les sous-champs
 // suivant, correspondant au champ identifie par aCD.
 // S'ilest precise, il suffit de prendre son contenu

 if (*(aCD->GetSubfield())==0)
  {
   TCDLib* Courant=this;
   *Temporary=0;
   while( Courant && *Courant==*aCD )
    {

     // Si le CDLib trouve n'identifie pas un sous-champ, on concatene, sans
     // mettre de balise de sous-champ

     if (*(Courant->GetSubfield())==0)
      strcat((char*)Temporary,Courant->GetContent());
     else

     // Si il y a un sous-champ d'identifie, il faut veiller a ne pas prendre
     // les indicateurs. Dans ce cas, on remet la balise marc et son contenu  

     if (*(Courant->GetSubfield())!='I')
      {
       sprintf((char*)Temporary2,"\x1F%c%s",*(Courant->GetSubfield()+1),Courant->GetContent());
       strcat((char*)Temporary,(char*)Temporary2);
      }


     Courant=(TCDLib*)Courant->GetNext();
    }
  }
 else strcpy((char *)Temporary,itsContent);
 
 delete aCD;

 // Temporary contient maintenant le contenu du CDLib.
 // On regarde maintenant l'intervalle de position a fournir

 int	MaxPos=strlen((char *)Temporary);
 int    aBegining=theCD->GetBegining();
 int    aEnd=theCD->GetEnd();


 // Si les positions de sont pas definies, on retourne tout le contenu
           
 if (aBegining==0 && aEnd==0) return (char*)Temporary;


 // Si les positions sont definies, on extrait les donnees correspondantes

 -- aBegining;
 if (aBegining>MaxPos || aEnd>MaxPos || (aEnd>0 && aBegining>aEnd))
  return strdup("");

 if (aEnd==-1)
  strcpy((char *)Temporary2,(char *)&Temporary[aBegining]);
 else
  {
   --aEnd;              
   memcpy((char *)Temporary2,(char *)&Temporary[aBegining],aEnd-aBegining+1);
   Temporary2[aEnd-aBegining+1]=0;             
  }
 return (char *)Temporary2;
} 

///////////////////////////////////////////////////////////////////////////////
//
// ResetContent
//
///////////////////////////////////////////////////////////////////////////////
void TCDLib::ResetContent(void)
{ 
 if (itsContent)	{ free(itsContent);	itsContent = NULL;	}
}


///////////////////////////////////////////////////////////////////////////////
//
// SetContent
//
// Cette methode remplit le contenu d'un CDLib. Deux cas sont possibles :
// - aCD est NULL, il faut simplement remplir le contenu du CDLib.
// - aCD defini in interval itsBegining -> itsEnd, le contenu passe doit servir
//   a mettre a jour ces positions uniquement.
//
// Le CDLib doit etre unitaire
//
///////////////////////////////////////////////////////////////////////////////
int 
TCDLib::SetContent(char *pContent, TCD* aCD)
{ 
  char* aContent;
  
  if (pContent)
   aContent=strdup(pContent);
  else 
   aContent=NULL;

  // Si aCD est NULL on remplit simplement le CDLib (test de validite a effectuer
  // en dehors de la fonction afin d'avoir le contexte de l'erreur).

  if (aCD==NULL)
  {
   if (*itsSubfield=='I' && strlen(aContent)>1)
    ErrorHandler->SetErrorD(5005,ERROR,aContent);
   if (itsContent) free(itsContent);
   itsContent=aContent;
   return 0;
  }


  // Si aCD n'est pas NULL, il faut verifier l'intervalle qu'il definit

  int    aBegining=aCD->GetBegining();
  int    aEnd=aCD->GetEnd();
  int    MaxPos;


  // Si les positions de debut et fin sont indefinies, on remplit simplement
  // le CDLib en re-appelant cette meme methode sans aCD (NULL par defaut)

  if (aBegining!=0 || aEnd!=0)

  // Sinon deux cas sont possibles :
  // - ou bien l'intervalle specifie "touche" ou chevauche le contenu actuel
  //   du champ, il suffit donc de completer le champ.
  // - ou bien cet intervalle ne "touche" pas le contenu actuel et on remplit 
  //   l'intervalle entre les deux par des blancs

   {
    // aBegining et aEnd commencent a 1 quand ils sont definis
    --aBegining; 

    if (itsContent)
     strcpy((char*)Temporary,itsContent);
    else
     *Temporary=0;
     
    MaxPos=strlen((char*)Temporary);
    for (int i=MaxPos;i<aBegining;Temporary[i++]=' ');

    if (*aContent)
     {
      if (aEnd==-1)
       {
        memcpy( (char*)&Temporary[aBegining], aContent, strlen(aContent));
        aEnd=aBegining+strlen(aContent)-1;
       }
      else
       {
        --aEnd;
        memcpy( (char*)&Temporary[aBegining], aContent, aEnd-aBegining+1);
       }
      if (MaxPos<=aEnd) Temporary[aEnd+1]=0;
     }
    SetContent((char*)Temporary);
   }
  else
   {
    SetContent(aContent);
   }
 free(aContent);

 return 0; 
}
   
///////////////////////////////////////////////////////////////////////////////
//
// IsEqual
//
/////////////////////////////////////////////////////////////////////////////// 
int TCDLib::IsEqual( TCD* aCD )
{
 // Le CDLib correspond au CD demande si :

 // 1 - Les Tags sont identiques
 if (strcmp(aCD->GetTag(),itsTag)!=0)
  return 0;

 // 2 - Le Subfield n'est pas specifie, ou les subfield sont identiques
 if (*(aCD->GetSubfield()))
  {
   if (strcmp(aCD->GetSubfield(),itsSubfield)!=0)
    return 0;
   
   // 3 - Le NS n'est pas specifie, ou ils sont identiques
   if (aCD->GetSubOccurenceNumber()>0 && aCD->GetSubOccurenceNumber()!=itsSubOccurenceNumber) 
    return 0;
  }

 // 4 - Le N n'est pas specifie, ou ils sont identiques
 if (aCD->GetOccurenceNumber()>0 && aCD->GetOccurenceNumber()!=itsOccurenceNumber)
  return 0;
  
 // 5 - Le NT n'est pas specifie, ou ils sont identiques
 if (aCD->GetTagOccurenceNumber()>0 && aCD->GetTagOccurenceNumber()!=itsTagOccurenceNumber) 
  return 0;

 return 1;
}

int
TCDLib::IsLess( TCD* aCD )
 {
  int rc;
  rc=strcmp(itsTag,aCD->GetTag());
  if (rc<0) return 1;
  if (rc>0) return 0;

  if (itsTagOccurenceNumber<aCD->GetTagOccurenceNumber()) return 1;
  if (itsTagOccurenceNumber>aCD->GetTagOccurenceNumber()) return 0;
  return 0;
 }


///////////////////////////////////////////////////////////////////////////////
//
// NextSubCDLib
//
// Cette methode decoupe le CDLib en sous-CDLibs unitaires, sur les balises de
// sous-champ
//
// Cette methode renvoie 1 si un sous-CDLib a ete trouve, 0 sinon
//
///////////////////////////////////////////////////////////////////////////////
int 
TCDLib::NextSubCDLib(TCDLib** pCDLib, int* _pos, char* defst)
 {
  TCDLib* aCDLib;
  int     first;
  int     pos=*_pos,d;
  char    tmp[3];

  tmp[0]='$';
  tmp[2]=0;
  first=(pos==0);

  // On parcourt tous les caracteres du contenu du CDLib, a partir de la position 
  // passee en parametre
 
  while(itsContent[pos])
   {
    if (itsContent[pos]==0x1F && itsContent[pos+1]!=0)
     {
     if (first && pos && *defst)
       {
        // Si on est a la premiere balise et qu'on n'est pas au debut du champ
        // et qu'un ss-champ par defaut est specifie, on le rempli avec le debut du champ
        tmp[1]=defst[1];
        memcpy((char*)Temporary,itsContent,pos);
        Temporary[pos]=0;
       }
      else
       {
        // Si on trouve une balise, on recopie le contenu du sous-champ dans
        // Temporary, jusqu'a la balise suivante

        tmp[1]=itsContent[pos+1];
        pos+=2;
        d=pos;
        while(itsContent[pos] && itsContent[pos]!=0x1F) ++pos;
        memcpy((char*)Temporary,&itsContent[d],pos-d);
        Temporary[pos-d]=0;
       }


      // On cree un nouveau CDLib correspondant au sous-champ trouve
      first=0;
      aCDLib=new TCDLib;
      aCDLib->SetTag(itsTag);
      aCDLib->SetTagOccurenceNumber(itsTagOccurenceNumber);
      aCDLib->SetSubfield(tmp);
      aCDLib->SetContent((char*)Temporary);
      if (!aCDLib->GetContent())
       ErrorHandler->SetErrorD(5000,ERROR,"While setting content of a new sub-cdlib");

      *pCDLib=aCDLib;
      *_pos=pos;
      return 1;      
     }
    ++pos;
   }

  *_pos=pos;
  return 0;
 }















