/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  tfile.cpp
 *
 *  implements a base class to manage file input./output
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h>
#include "tfile.h"                 
#include "error.h"
#include  "xvtstuff.h"

///////////////////////////////////////////////////////////////////////////////
//
// TFile
//
///////////////////////////////////////////////////////////////////////////////   

TFile::TFile( FILE_SPEC *FileInfo, char Mode, char Kind)
 {
  itsFileInfo	= FileInfo;
  itsMode		= Mode;
  itsKind		= Kind;
  itsLineNumber = 0;          
                       
  Included		= NULL;                       
  File			= NULL;
  iFile			= -1;
 }

  

///////////////////////////////////////////////////////////////////////////////
//
// TFile
//
///////////////////////////////////////////////////////////////////////////////   

TFile::~TFile()
 {

 }
 
///////////////////////////////////////////////////////////////////////////////
//
// Open
//
/////////////////////////////////////////////////////////////////////////////// 

int TFile::Open()
 {
  char	mode[2];

  xvt_fsys_set_dir(itsFileInfo->dir);   
  if (itsKind==FILE_ASCII)
   {
    sprintf(mode,"%c",itsMode);
    if ((File=fopen(itsFileInfo->name, mode))==NULL)
     {

		xvt_fsys_set_dir_startup();
      return ErrorHandler->SetErrorD(9501,WARNING,itsFileInfo->name);
     }
   }
  else
   {
    if (itsMode==FILE_READ)
     iFile=::open(itsFileInfo->name,O_RDONLY|O_BINARY,0640);
    else
     iFile=::open(itsFileInfo->name,O_CREAT|O_WRONLY|O_TRUNC|O_BINARY,0640);

    xvt_fsys_set_dir_startup();
    if (iFile == -1)
     return ErrorHandler->SetErrorD(9502,WARNING,itsFileInfo->name);
   }
   
  xvt_fsys_set_dir_startup();
  Included=NULL;
  if (itsMode==FILE_READ && itsKind==FILE_ASCII)
   return SkipBegining();
  else
   return 0;
 } 
 
///////////////////////////////////////////////////////////////////////////////
//
// Close
//
///////////////////////////////////////////////////////////////////////////////   
int TFile::Close()
 {
  if (itsKind==FILE_ASCII)
   fclose(File);
  else          
   close(iFile);
  File=NULL;
  iFile=-1;
  return 0;
 } 
   
///////////////////////////////////////////////////////////////////////////////
//
// Exists
//
///////////////////////////////////////////////////////////////////////////////   
BOOLEAN TFile::Exists()
 {
  return (BOOLEAN)xvt_fsys_get_file_attr(itsFileInfo,XVT_FILE_ATTR_EXIST);
 }  
   

///////////////////////////////////////////////////////////////////////////////
//
// NextLine
//
///////////////////////////////////////////////////////////////////////////////
int TFile::NextLine(char *aLine, int aMaxLine, char *aComment, int aMaxComment, FILE_SPEC* Spec, int *LineNumber)
 {
  char		tempo[(10*BUFF_SIZE)+1];
  FILE_SPEC	IncludedFile;
  int		i,
  		rc;
  
  memset(tempo, 0, (10*BUFF_SIZE)+1);
  // NextLine() can't operate on binary files 
  if (itsKind==FILE_BINARY)
   return ErrorHandler->SetErrorD(9504,ERROR,itsFileInfo->name);

  if (Included!=NULL) 
  // current line is an #include line
   {
    if ((rc=Included->NextLine(aLine,aMaxLine,aComment,aMaxComment,Spec,LineNumber))<=0)
     return rc;
    else
     {
      delete Included;
      Included=NULL;
     }
   }
   
  if (Spec)
   memcpy(Spec,itsFileInfo,sizeof(FILE_SPEC));

  if (fgets( tempo, 10*BUFF_SIZE, File )==NULL)
   return 1;
   
  ++itsLineNumber;
  if (LineNumber) *LineNumber=itsLineNumber;   
    
  // The current line begins with the '#define' word  
  if (!memcmp(tempo,"#include",8))
   {
    // looking for the name of the included file
    i=9;
    while (tempo[i] && tempo[i]!='"')
     ++i;
    if (tempo[i]!='"')
    // '#define' directive not folowed by a file name
     {
      sprintf((char *)Temporary,"%s : %d",Spec->name,itsLineNumber);
      ErrorHandler->SetErrorD(9505,WARNING,(char *)Temporary);
     }
    else
     {
      int j=++i;
      while(tempo[j])
       if (tempo[j]=='"') break;
       else ++j;
      tempo[j]=0;
      memcpy( &IncludedFile, itsFileInfo, sizeof(FILE_SPEC));
      strcpy( IncludedFile.name, &tempo[i]);
      Included=new TFile(&IncludedFile,FILE_READ,FILE_ASCII);
      rc=Included->Open();
      if (rc<0)
       return rc;
      rc=Included->NextLine(aLine,aMaxLine,aComment,aMaxComment,Spec,LineNumber);
      if (rc<=0)
       return rc;
      else
       {
        delete Included;
        Included=NULL;
       }
     }
   }
  i=0;
  if (tempo[strlen(tempo)-1]=='\n')
   tempo[strlen(tempo)-1]=0;
  while( tempo[i] )
   {
    if (tempo[i]=='/' && tempo[i+1]=='/')
     break;
    ++i;
   }
   
  if (aComment)
   {
    if (tempo[i])
     {
      int j=i+2;
      while(tempo[j] && tempo[j]==' ')
       ++j;
      if ((int)strlen(tempo)>aMaxComment-1)
       {
        sprintf((char *)Temporary," in file '%s' at line '%d' (%s)",Spec->name,*LineNumber,&tempo[i]);
        ErrorHandler->SetErrorD(510,DISPLAY,(char *)Temporary); 
       } 
      memcpy(aComment,&tempo[j],aMaxComment-1);
      aComment[aMaxComment-1]=0;
     }
    else *aComment=0;
   }
  tempo[i]=0;
  if ((int)strlen(tempo)>aMaxLine-1) 
   {
    sprintf((char *)Temporary," in file '%s' at line '%d' (%s)",Spec->name,*LineNumber,tempo);
    ErrorHandler->SetErrorD(511,DISPLAY,(char *)Temporary); 
   }   
//  memcpy(aLine,tempo,aMaxLine-1);
//  aLine[aMaxLine-1]=0;   
  strcpy(aLine,tempo);

  return 0;
 }
   
   
///////////////////////////////////////////////////////////////////////////////
//
// GetName
//
///////////////////////////////////////////////////////////////////////////////
FILE_SPEC *TFile::GetName(void)   
{
 return itsFileInfo;
} 

///////////////////////////////////////////////////////////////////////////////
//
// SetName
//
///////////////////////////////////////////////////////////////////////////////
void TFile::SetName(FILE_SPEC *aName)   
{
 itsFileInfo	= aName	;
} 

///////////////////////////////////////////////////////////////////////////////
//
// GetMode
//
///////////////////////////////////////////////////////////////////////////////
char TFile::GetMode(void)   
{
 return itsMode;
} 

///////////////////////////////////////////////////////////////////////////////
//
// SetMode
//
///////////////////////////////////////////////////////////////////////////////
void TFile::SetMode(char aMode)   
{
 itsMode	= aMode;
} 

///////////////////////////////////////////////////////////////////////////////
//
// GetKind
//
///////////////////////////////////////////////////////////////////////////////
char TFile::GetKind(void)   
{
 return itsKind;
} 

///////////////////////////////////////////////////////////////////////////////
//
// SetKind
//
///////////////////////////////////////////////////////////////////////////////
void TFile::SetKind(char aKind)   
{
 itsKind	= aKind;
}
   
///////////////////////////////////////////////////////////////////////////////
//
// GetSize
//
///////////////////////////////////////////////////////////////////////////////
long TFile::GetSize(void)   
{ 
 long	OldPosition,
 		EndPosition;
 
 if (itsKind==FILE_BINARY)
  {
   OldPosition=_lseek(iFile,0,SEEK_CUR);
   EndPosition=_lseek(iFile,0,SEEK_END);
   _lseek(iFile,OldPosition,SEEK_SET);
  }
 else
  {
   OldPosition=ftell(File);
   fseek(File,0,SEEK_END);
   EndPosition=ftell(File);
   fseek(File,OldPosition,SEEK_SET);
  } 
   
 return EndPosition;
}   

///////////////////////////////////////////////////////////////////////////////
//
// GetPos
//
///////////////////////////////////////////////////////////////////////////////
long TFile::GetPos(void)   
{ 
 if (itsKind==FILE_BINARY)
  return _lseek(iFile,0,SEEK_CUR);
 else
  return ftell(File);
}


///////////////////////////////////////////////////////////////////////////////
//
// SkipBegining
//
///////////////////////////////////////////////////////////////////////////////
int
TFile::SkipBegining()
 {
  FILE_SPEC		aSpec;
           
  ErrorHandler->Reset();       
           
  // Au debut de chaque fichier, il faut sauter deux lignes de commentaires
  if (NextLine(itsFirstLine,BUFF_SIZE,NULL,0,&aSpec))
   ErrorHandler->SetErrorD(9506,ERROR,aSpec.name);
  if(NextLine(itsSecondLine,BUFF_SIZE,NULL,0,&aSpec))
   ErrorHandler->SetErrorD(9506,ERROR,aSpec.name);
  return ErrorHandler->GetErrorCode();
 }

