/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  trule.cpp
 *
 *  implements a class to manage rules
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h> 
#include "trule.h"       
#include "error.h"
#include "tools.h"

///////////////////////////////////////////////////////////////////////////////
//
// TRule
//
///////////////////////////////////////////////////////////////////////////////  
TRule::TRule(TCD *LastInputCD, TCD *LastOutputCD)
{
 itsInputCD		= NULL;
 itsOutputCD		= NULL;
 itsLib			= NULL;
 itsComment		= NULL;
 itsNextRule		= NULL;
 itsPreviousRule	= NULL;
 itsLastInputCD		= LastInputCD;
 itsLastOutputCD	= LastOutputCD;  
}  

///////////////////////////////////////////////////////////////////////////////
//
// TRule copy constructor
//
///////////////////////////////////////////////////////////////////////////////  
TRule::TRule(TRule *aRule)
{
 itsInputCD			= new TCD(aRule->GetInputCD());
 itsOutputCD		= new TCD(aRule->GetOutputCD());
 itsLib				= NULL;
 SetLib(aRule->GetLib());
 itsComment			= NULL;
 SetComment(aRule->GetComment());
 itsNextRule		= aRule->GetNextRule();
 itsPreviousRule	= aRule->GetPreviousRule();
 itsLastInputCD		= aRule->itsLastInputCD;
 itsLastOutputCD	= aRule->itsLastOutputCD;  
}

///////////////////////////////////////////////////////////////////////////////
//
// ~TRule
//
///////////////////////////////////////////////////////////////////////////////   
TRule::~TRule()
{ 
 if (itsInputCD)	{ delete itsInputCD;	itsInputCD	= NULL;	}
 if(itsOutputCD)	{ delete itsOutputCD;	itsOutputCD	= NULL;	}
 if (itsLib)		{ free(itsLib);		itsLib 		= NULL; }
 if (itsComment)	{ free(itsComment);	itsComment	= NULL;	}
}
          
///////////////////////////////////////////////////////////////////////////////
//
// SetLib
//
///////////////////////////////////////////////////////////////////////////////   
int TRule::SetLib(char *aLib)          
{
 int	CurrentPos,
 		MaxPos;
    
 if ((!aLib) || (!(*aLib)))
  return 0;  
    
 MaxPos=strlen(aLib);   
 for (CurrentPos=0; CurrentPos<MaxPos; CurrentPos++)
  if (aLib[CurrentPos]=='\t')
   aLib[CurrentPos]=' ';
  
 if (itsLib)
  {
   strcpy((char *)Temporary2,itsLib);
   strcat((char *)Temporary2,"\n");
   strcat((char *)Temporary2,aLib);
   free(itsLib);
   itsLib=NULL;
  }            
 else
  strcpy((char *)Temporary2,aLib);
 itsLib=strdup((char *)Temporary2);
 if (!itsLib)
  return ErrorHandler->SetError(5502,ERROR);
 return 0;   
}

///////////////////////////////////////////////////////////////////////////////
//
// SetComment
//
///////////////////////////////////////////////////////////////////////////////   
int TRule::SetComment(char *aComment)
{
 int	iIndice;
    
 if ((!aComment) || (!(*aComment)))
  return 0;
 if (itsComment)
  {  
   strcpy((char *)Temporary2,itsComment);
   strcat((char *)Temporary2,"\n");
   strcat((char *)Temporary2,aComment);
   free(itsComment);
   itsComment=NULL;
  }            
 else
  strcpy((char *)Temporary2,aComment);
 itsComment=strdup((char *)Temporary2);
 if (!itsComment)
  return ErrorHandler->SetError(5503,ERROR);
  
 // Suppression des Tabulations 
 for (iIndice=0; iIndice<(int)strlen(itsComment); iIndice++)
  if (itsComment[iIndice]=='\t')
   itsComment[iIndice]=' ';
 return 0;
} 
          
///////////////////////////////////////////////////////////////////////////////
//
// FromString
//
///////////////////////////////////////////////////////////////////////////////   
int TRule::FromString(char *aString, char *aComment)
{
 char		*ItemPointer;
 int		NbPipes,
 			MaxPos,
 			CurrentPos,
 			RealPos=0;
 int		ReturnCode;			
                   
 MaxPos=strlen(aString);        
 
 if ((!(*aString)) && (!(*aComment)))
 // empty lines are ignored
  return 0;  
                               
 // Count the number of '|' caracters                              
 for (CurrentPos=RealPos,NbPipes=0; CurrentPos<MaxPos; CurrentPos++) {
	 if (aString[CurrentPos]=='|') {
		NbPipes++;
	 }
 }
   
 if (NbPipes>2)
  NbPipes=2;
 switch(NbPipes)
  {
   case 2 : // the rule is a complete rule ( CDin | CDOut | Rule ) 
    if ((ItemPointer=(char *)strtok(aString,"|"))!=NULL)
     if (RemoveSpace(ItemPointer))
     // the CDIn is not empty
      {                 
       itsInputCD = new TCD;
       if (!itsInputCD)
        return -5504;
       if ((ReturnCode=itsInputCD->FromString(ItemPointer,itsLastInputCD,INPUT))!=0)
        return -ReturnCode;
      }
   case 1 : // the rule has no CDIn or has an empty CDIn
    // set the CDIn with the CDIn of the previous Rule ( itsLastInputCD )
    if (!itsInputCD)
     {                 
      itsInputCD = new TCD;
      if (!itsInputCD)
       return -5504; 
      if ((ReturnCode=itsInputCD->FromString("",itsLastInputCD,INPUT))!=0)
       return -ReturnCode;
     }  
    if (NbPipes==1)
    // the Rule is a partial rule ( CDOut | Rule )
     ItemPointer=(char *)strtok(aString,"|");
    else
    // the Rule is a complete Rule with an empty CDIn 
     ItemPointer=(char *)strtok(NULL,"|");
    if (ItemPointer)
     {
      itsOutputCD = new TCD; 
      if (!itsOutputCD)
       return -5504;
      RemoveSpace(ItemPointer);
      if ((ReturnCode=itsOutputCD->FromString(ItemPointer,itsLastOutputCD,OUTPUT))!=0)
       return -ReturnCode;
     }                              
   case 0:
    if (NbPipes==0)
    // This line is a new part of the previous Rule
     {
      if (RemoveSpace(aString,AT_BEGINING|AT_END))
       if (SetLib(aString))
        return -5502;
      if (RemoveSpace(aComment,AT_BEGINING|AT_END))  
       if (SetComment(aComment))
        return -5503;
      return 0;
     }
    else 
     {
      if ((ItemPointer=(char *)strtok(NULL,"~")))
       if (RemoveSpace(ItemPointer,AT_BEGINING|AT_END)) 
        if (SetLib(ItemPointer))
         return -5502;
     }
    break;
   default:
    break;        
  }  
 if (RemoveSpace(aComment,AT_BEGINING|AT_END))  
  if (SetComment(aComment))
   return -5503;
   
 return NbPipes;
} 

///////////////////////////////////////////////////////////////////////////////
//
// ToString
//
///////////////////////////////////////////////////////////////////////////////   
#define TAB				' '
#define START_RULE_AT	30
int TRule::ToString(char *aString, char *aComment)
{               
 int	CurrentPos=0;
 int	iIndice;
 int	iLng;
 char	Separateur[5];
 char	szStart[3+START_RULE_AT+1];
 
 memset(szStart,' ',3+START_RULE_AT);
 szStart[3+START_RULE_AT]=0;
 sprintf(Separateur,"%c| ",TAB);

 *aString=NULL;

 if ((!itsInputCD) || (!itsOutputCD))
  return 1;
  
 if (itsInputCD->ToString(&aString[CurrentPos],INPUT))
 // Absence du Tag d'entre
  return 1;
 CurrentPos = strlen(aString);
 
 strcat(aString,Separateur);
 CurrentPos += strlen(Separateur);
 
 if (itsOutputCD->ToString(&aString[CurrentPos],OUTPUT))
 // Absence du Tag de sortie
  return 1;
 CurrentPos = strlen(aString);
 
 if (CurrentPos<START_RULE_AT)
  {
   for (iIndice=CurrentPos; iIndice<START_RULE_AT; iIndice++)
    aString[iIndice]=' ';
   CurrentPos=iIndice; 
   aString[CurrentPos]=0;
  } 
   
 strcat(aString,Separateur);
 CurrentPos += strlen(Separateur);
   
 if (itsLib)
  {
   strcat(aString,itsLib);
   iLng=strlen(aString);
   for (iIndice=0; iIndice<iLng; iIndice++)
    if (aString[iIndice]=='\n')
   iIndice+=InsertChar(aString,&iLng,iIndice+1,szStart,APRES);   
   CurrentPos = strlen(aString);
  }             
  
 if (itsComment) 
  { 
   sprintf(aComment,"// %s",itsComment);
   iLng=strlen(aComment);
   for (iIndice=0; iIndice<iLng; iIndice++)
    if (aComment[iIndice]=='\n')
   iIndice+=InsertChar(aComment,&iLng,iIndice+1,"// ",APRES);
  } 
 else
  *aComment = NULL;
         
 return 0;
}  



















