/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  lien_not.cpp
 *
 *  a bunch of functions to manage strings and interface to the
    lexical analysis
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 *  Rev history:
 *  17 Aug 2000 -  version raised to v1.1 - use of temporary file name now
    uses the tmpfile() function instead of tmpnam().  This is so that the temporary
	files created are deleted on program exit.

 */

#include <afx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "error.h"
#include "tcdlib.h"
#include "sort.h"
#include "umrecord.h"
#include "trule.h"
#include "codedata.h"


extern TUMApplication *g_theApplication;

// John Hough addition
extern TypeInst *g_D;
TypeInst DStruct;

#ifndef _UNIX_
extern "C" {
#endif

// Variables globales necessaires a l'echange de donnees avec l'analyseur yacc
#include "typedef.h"
extern char yytext[];
extern FILE* yyin;

TypeInst* S;
TypeInst* T;
TypeInst* D;
TypeInst* N;
TypeInst* NT;
TypeInst* NS;
TypeInst* NO;
TypeInst* NTO;
TypeInst* NSO;
TypeCD* CDIn;

TypeInst* Memoire[100];

int Eval;
int Error;
char* RedoStr=NULL;
char* MainInput=NULL;
int AfterRedo=0, Fini=0;
int RedoFlag;
extern unsigned long ordinal;

#include "fonc.h"

char* NextSubField_C(TypeCD*,TypeCD*);
char* LastSubField_C(TypeCD*,TypeCD*);
int   Precedes_C(TypeCD*,TypeCD*);
int   Exists_C(TypeCD*);
char* LireCD_C(TypeCD*);

#ifndef _UNIX_
}
#endif


TUMRecord* InputRecord, *OutputRecord;
TCDLib*    InputCDL;
TCD*	   theCDOut;
TRule*     CurrentRule;
TRuleDoc*  RuleDoc;
static SortElem* ListSort=NULL;
SortElem*  LastSort;

char TmpRule[80];


// Fonctions globales necessaires a l'echange de donnees avec l'analyseur yacc

#ifndef _UNIX_
extern "C" {
#endif

extern char* ToString( TypeInst* );
extern int FreeMem( TypeInst* );
extern int yyparse(void);
int yywrap(void)
 {
  return 1;
 }


// Cette fonction est appellee par l'analyseur grammatical en cas d'erreur

void yyerror( const char* err )
  {
    if (*err=='s' || *err=='l')
	sprintf((char*)Temporary,"%s near `%s' in rule \"",err,yytext);
    else
	sprintf((char*)Temporary,"ERROR `%s' in rule ",err);
    ++Error;
    CurrentRule->ToString((char*)&Temporary[strlen((char*)Temporary)],(char*)Temporary2);
    strcat((char*)Temporary,"\"");
    ErrorHandler->SetErrorD(5100,ERROR,(char*)Temporary);
  }

#ifndef _UNIX_
}
#endif


// Cette fonction permet de savoir si une regle commence par un +

int
IsConcat(char* lib)
 {
  if (lib==NULL) return 0;
  char*ptr=lib;
  while(*ptr && *ptr==' ') ++ ptr;
  if (*ptr=='+') return 1; else return 0;
 }


// Cette fonction remplace les valeurs symboliques de n,ns et nt par leur
// valeur reelle

int
Replace_N_NT_NS( int val, int N, int NT, int NS )
 {
   switch(val)
   {
    case CD_N  : return N;
    case CD_NT : return NT;
    case CD_NS : return NS;
    default    : return val;
   }
 }


void
ResetRedo()
 {
  AfterRedo=Fini=RedoFlag=0;
  if (RedoStr) free(RedoStr);
  RedoStr=NULL;
  if (MainInput) free(MainInput);
  MainInput=NULL;
 }  


int
Init_Evaluate_Rule(void *, int Mode)
 {
  RuleDoc=g_theApplication->itsRuleDoc;
  int i;
  S=(TypeInst*)malloc(sizeof(TypeInst));S->str=NULL;
  D=(TypeInst*)malloc(sizeof(TypeInst));D->str=NULL;
  N=(TypeInst*)malloc(sizeof(TypeInst));N->str=NULL;
  NT=(TypeInst*)malloc(sizeof(TypeInst));NT->str=NULL;
  NS=(TypeInst*)malloc(sizeof(TypeInst));NS->str=NULL;
  NO=(TypeInst*)malloc(sizeof(TypeInst));NO->str=NULL;
  NTO=(TypeInst*)malloc(sizeof(TypeInst));NTO->str=NULL;
  NSO=(TypeInst*)malloc(sizeof(TypeInst));NSO->str=NULL;
  CDIn=(TypeCD*)malloc(sizeof(TypeCD));CDIn->Fixed=NULL;
  if (!S ||!D ||!N || !NT || !NS || !NO || !NSO || !NTO || !CDIn)
   ErrorHandler->SetErrorD(5000,ERROR,"When initialising variables");

  // -------------------------------------------------
  // Initialisation du fichier de communication des regles :
  // pour evaluer les regles par lex&yacc, il faut les ecrire dans 
  // un fichier. Le fichier suivant est ouvert avec une taille de bloc
  // normalement suffisante pour que la regle ne soit jamais physiquement
  // flushe sur disque

  if (yyin==NULL/*stdin*/)
   {
//    tmpnam(TmpRule);
//    yyin=fopen(TmpRule,"w+");
	// rev 1.1 mod below
	yyin = tmpfile();			// returns an open file name
	// end rev 1.1 mod

    // Permet de ne pas ecrire physiquement sur disque le fichier
    setvbuf( yyin, NULL, _IOFBF, 2048 );   
 
    // On initialise les memoires. Elles ne sont pas remises a zero entre chaque
    // notice, ce qui permet de recuperer des infos des notices precedantes par exemple.   
    for (i=0;i<100;++i)
     {
      Memoire[i]=(TypeInst*)malloc(sizeof(TypeInst));
      if (!Memoire[i])
       ErrorHandler->SetErrorD(5000,ERROR,"When initialising memories");
      Memoire[i]->val=0;
      Memoire[i]->str=NULL;
     }  
 
   }

  NextSubField=&NextSubField_C;
  LastSubField=&LastSubField_C;
  Precedes=&Precedes_C;
  Exists=&Exists_C;
  LireCD=&LireCD_C;

  return 0;
 }




//
// Cette fonction effectue la conversion d'un UMRecord en un autre, conformement
// a une regle.
//

int 
Evaluate_Rule( TUMRecord* In, TUMRecord* Out, TRule* Rule )
 {
  int rc,concr;


  // -------------------------------------------------
  // Renseignement des variables globales
  InputRecord=In;
  OutputRecord=Out;
  CurrentRule=Rule;
 



  // Ecriture de la regle dans le fichier de communication
  fseek(yyin,0L,0);
  if (Rule->GetLib())
   fprintf(yyin,"%s\n#",Rule->GetLib());
  else
   fprintf(yyin,"S\n#");


  // -------------------------------------------------
  // On parcourt tous les CDLib qui correspondent au CD en entree de la regle
  TCDLib* aCDLIn=In->GetFirstCDLib();
  while( In->NextCD(&aCDLIn, Rule->GetInputCD()) )
   {
    // Initialisation de la memoire pour les variables de l'analyse de regles
    if (!S) S=(TypeInst*)malloc(sizeof(TypeInst));S->str=NULL;
    if (!D) D=(TypeInst*)malloc(sizeof(TypeInst));D->str=NULL;
    if (!S ||!D ||!N || !NT || !NS || !NO || !NSO || !NTO || !CDIn)
      ErrorHandler->SetErrorD(5000,ERROR,"When initialising variables");

    // Initialisation de S,N,NT,NS : valeurs de l'entree
    N->str=NT->str=NS->str=NULL;
    N->val=aCDLIn->GetOccurenceNumber();
    NT->val=aCDLIn->GetTagOccurenceNumber();
    if (*(Rule->GetInputCD()->GetSubfield()))
     NS->val=aCDLIn->GetSubOccurenceNumber();
    else
     NS->val=0;
    S->val=0;
    if (S->str) free(S->str);
    S->str=(char*)strdup(aCDLIn->GetContent(Rule->GetInputCD()));   
    if (!S->str)
     ErrorHandler->SetErrorD(5000,ERROR,"When allocating S content");
    InputCDL=aCDLIn;

    // Initialisation des parametres du CDIn
    strcpy(CDIn->Field,aCDLIn->GetTag());
    strcpy(CDIn->SubField,aCDLIn->GetSubfield());
    CDIn->nt=aCDLIn->GetTagOccurenceNumber();
    CDIn->ns=aCDLIn->GetSubOccurenceNumber();  


    // -------------------------------------------------
    // Boucle d'evaluation de la regle
    do
     {  
      // On cree une copie du CDOut (on va peut-etre le modifier)
      TCD* aCDOut=new TCD(Rule->GetOutputCD());

      // Si n, ns ou nt apparaissent dans CDOut, on les remplace par leur valeur
      aCDOut->SetOccurenceNumber(
              Replace_N_NT_NS(aCDOut->GetOccurenceNumber(), N->val, NT->val, NS->val) );
      aCDOut->SetTagOccurenceNumber(
              Replace_N_NT_NS(aCDOut->GetTagOccurenceNumber(), N->val, NT->val, NS->val) );
      if (*(aCDOut->GetSubfield())=='I')
       aCDOut->SetSubOccurenceNumber(1);
      else
       aCDOut->SetSubOccurenceNumber(
              Replace_N_NT_NS(aCDOut->GetSubOccurenceNumber(), N->val, NT->val, NS->val) );


	  

      NO->str=NTO->str=NSO->str=NULL;
      
	  if (D) FreeMem(D);
	  
	  //John Hough - replaced malloc with direct structure
	  //D = &DStruct;
	  //D->str = NULL;
	   //memset(D, 0, sizeof(TypeInst));

      D=(TypeInst*)malloc(sizeof(TypeInst));

	  // John Hough - added g_D
		g_D = D;
 
      D->str=NULL;
      TCDLib* aCDLOut=Out->GetLastCDLib();


      // On recherche le CDLib correspondant dans la notice de sortie
      // Initialisation de D : valeur de la sortie

      if (Out->PreviousCD(&aCDLOut, aCDOut))
       {

        // On a trouve un CDLib correspondant. Deux cas sont possibles : 
        // Soit un subfield existe, on renseigne NO, NSO, NTO
        // sinon, nous sommes a la fin d'un champ, il n'y a pas de sous-champ
        // de defini, donc NSO=0 et NO=NTO

        D->val=0;
        if (*(aCDOut->GetSubfield()))
         {
          NO->val=aCDLOut->GetOccurenceNumber();
          NTO->val=aCDLOut->GetTagOccurenceNumber();
          NSO->val=aCDLOut->GetSubOccurenceNumber();
         }
        else
         {
          NSO->val=0;
          NO->val=NTO->val=aCDLOut->GetTagOccurenceNumber();


          // On va se positionner au debut de ce CDLib representant un champ complet
          TCDLib* Courant=(TCDLib*)aCDLOut->GetPrevious();
          TCD ToSearch;
          ToSearch.SetTag(aCDLOut->GetTag());
          ToSearch.SetTagOccurenceNumber(aCDLOut->GetTagOccurenceNumber());
          while( Courant )
           if (*Courant==ToSearch ) Courant=(TCDLib*)Courant->GetPrevious();
           else break;
          if (Courant)
           aCDLOut=(TCDLib*)Courant->GetNext();
          else
           aCDLOut=Out->GetFirstCDLib();
         }
 

        // Si la regle ne contient pas de + au debut, il faut regarder si des no,nto ou nso
        // apparaissent pour les incrementer

        if (!IsConcat(Rule->GetLib()))
         {
          concr=0;

          // Cas particuliers :
          // TTT(no)SS
          if (aCDOut->GetTagOccurenceNumber()==CD_NO) 
           {
           if (aCDOut->GetSubOccurenceNumber()<0)
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5101,ERROR,(char*)Temporary);
             }
            NTO->val=++NO->val;

            D->str=(char *)strdup("");
           }
          else
          // TTTSS(no)
          if (aCDOut->GetSubOccurenceNumber()==CD_NO)
           {
            if (aCDOut->GetTagOccurenceNumber()<=0) 
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5102,ERROR,(char*)Temporary);
             }
            NSO->val=++NO->val;
            D->str=(char*)strdup("");
           }
          else
          // TTT(nto)SS
          if (aCDOut->GetTagOccurenceNumber()==CD_NTO) 
           {
            if (aCDOut->GetSubOccurenceNumber()<0) 
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5103,ERROR,(char*)Temporary);
             }
            ++NTO->val; ++NO->val; 

            D->str=(char *)strdup("");
           }
          else
          // TTTSS(nso)
          if (aCDOut->GetSubOccurenceNumber()==CD_NSO) 
           {
            if (aCDOut->GetTagOccurenceNumber()<=0) 
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5104,ERROR,(char*)Temporary);
             }
            ++NSO->val; ++NO->val; 
            D->str=(char*)strdup("");
           }
          else

           D->str=(char *)strdup("");
         }
        else
	 {
          concr=1;
          D->str=(char*)strdup(aCDLOut->GetContent(aCDOut));
	  // Dans le cas de concatnation sur un champ complet,
	  // il faut supprimer le champ de la notice de sortie,
	  // sinon on ajoutera sans concatner
	  //if (NTO->val==0)
	  //  Out->DeleteCDLib(aCDOut);
	 }
       }
      else
       {
        // On n'a pas trouve de CDLib existant qui corresponde
        D->val=0;

        D->str=(char *)strdup("");

        NO->val=aCDOut->GetOccurenceNumber();
        NSO->val=aCDOut->GetSubOccurenceNumber();
        NTO->val=aCDOut->GetTagOccurenceNumber();
        concr=0;

        // On recherche les donnees sur no,nso et nto qui nous manquent 
        // Cas particuliers :
        // TTT(no)SS
        if (aCDOut->GetTagOccurenceNumber()==CD_NO) 
         {
           if (aCDOut->GetSubOccurenceNumber()<0)
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5101,ERROR,(char*)Temporary);
             }
          NTO->val=NO->val=1;
         }
        else
        // TTTSS(no)
        if (aCDOut->GetSubOccurenceNumber()==CD_NO)
         {
           if (aCDOut->GetTagOccurenceNumber()<=0)
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5102,ERROR,(char*)Temporary);
             }
          NSO->val=NO->val=1;
         }
        else
        // TTT(nto)SS
        if (aCDOut->GetTagOccurenceNumber()==CD_NTO) 
         {
           if (aCDOut->GetSubOccurenceNumber()<0)
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5103,ERROR,(char*)Temporary);
             }
          NTO->val=1;
         }
        else
        // TTTSS(nso)
        if (aCDOut->GetSubOccurenceNumber()==CD_NSO) 
         {
           if (aCDOut->GetTagOccurenceNumber()<=0)
             { 
               Rule->ToString((char*)Temporary,(char*)Temporary2);
               ErrorHandler->SetErrorD(5104,ERROR,(char*)Temporary);
             }
          NSO->val=1;
         }
      }
 
	  

      aCDOut->SetTagOccurenceNumber(NTO->val);
      aCDOut->SetSubOccurenceNumber(NSO->val);
       

      // Si NO n'est pas renseigne, il faut le faire

      if (NO->val<=0)
       {
        TCDLib* Search=Out->GetLastCDLib();
        TCD aCD(aCDOut);
        aCD.SetTagOccurenceNumber(0);
        aCD.SetSubOccurenceNumber(0);


        // On recherche le dernier CD correspondant

        if (Out->PreviousCD(&Search,&aCD))
         {
          if (*(Search->GetSubfield())=='$')
           NO->val=Search->GetOccurenceNumber()+1;
          else
           NO->val=0;
         }
        else
         NO->val=1;
       }
      aCDOut->SetOccurenceNumber(NO->val);
 


      Error=0;
      RedoFlag=0;
      fseek(yyin,0L,0);
      theCDOut=aCDOut;
      rc=yyparse();


      if (rc!=2)
       {
        TCDLib aCDLOut(aCDOut);
        aCDLOut.SetBegining(0);
        aCDLOut.SetEnd(0);

		// John Hough
		// added ptr, replaced ToString(S) with ptr 
		// and delete 
		char *ptr = ToString(S);
        aCDLOut.SetContent(ptr);
        
		if (ptr)
			free(ptr);
		// end of John Hough add
		Out->InsereCDLib(&aCDLOut,aCDOut,concr);
       }
      if (T)
       {
        FreeMem(S);
        S=T;
	T=NULL;
       }
      delete aCDOut;
     }
    while( rc==3 );
  
	



   // On passe au champ suivant en entree 

   TCD Courant(Rule->GetInputCD());
   Courant.SetTagOccurenceNumber(NT->val);
   Courant.SetSubOccurenceNumber(NS->val);

   while (aCDLIn)
     {
      if (*aCDLIn!=Courant) break;
      aCDLIn=(TCDLib*)aCDLIn->GetNext();
     } 
  }



 return ErrorHandler->GetErrorCode();
}






int End_Evaluate_Rule()
 {
 int i;
 // Liberation de la memoire
 for (i=0;i<100;++i) 
  if (Memoire[i]) 
   {
    FreeMem(Memoire[i]);
    Memoire[i]=NULL;
   }

// if (S) FreeMem(S); 
  S=NULL;
 if (T) FreeMem(T); T=NULL;
 
 // John Hough - removed because of conflicts
 if (D) FreeMem(D); D=NULL;
 
 if (N) FreeMem(N); N=NULL;
 if (NT) FreeMem(NT); NT=NULL;
 if (NS) FreeMem(NS); NS=NULL;
 if (NO) FreeMem(NO); NO=NULL;
 if (NTO) FreeMem(NTO); NTO=NULL;
 if (NSO) FreeMem(NSO); NSO=NULL;
 if (CDIn)
  {
   if (CDIn->Fixed) free(CDIn->Fixed);
   free(CDIn);
  }

  
 CDIn=NULL;

 return 0; 
 }


void FinishCD(TypeCD* aCD)
 {
  if (!aCD)
   return;
  switch(aCD->nt)
   {
    case CD_N   : aCD->nt=N->val; break;
    case CD_NS  : aCD->nt=NS->val; break;
    case 0 :
    case CD_NT  : aCD->nt=NT->val; break;
    case CD_NO  : aCD->nt=NO->val; break;
    case CD_NSO : aCD->nt=NSO->val; break;
    case CD_NTO : aCD->nt=NTO->val; break;
   }
  switch(aCD->ns)
   {
    case CD_N   : aCD->ns=N->val; break;
    case CD_NS  : aCD->ns=NS->val; break;
    case CD_NT  : aCD->ns=NT->val; break;
    case CD_NO  : aCD->ns=NO->val; break;
    case CD_NSO : aCD->ns=NSO->val; break;
    case CD_NTO : aCD->ns=NTO->val; break;
   }
 }

void FinishTCD(TypeCD* aCD)
 {
  if (!aCD)
   return;
  switch(aCD->nt)
   {
    case CD_N   : aCD->nt=N->val; break;
    case CD_NS  : aCD->nt=NS->val; break;
    case 0 :
    case CD_NT  : aCD->nt=NT->val; break;
    case CD_NO  : aCD->nt=NO->val; break;
    case CD_NSO : aCD->nt=NSO->val; break;
    case CD_NTO : aCD->nt=NTO->val; break;
   }
 }

void PrCD( TypeCD* CD )
 {
  if (CD)
   printf("%s(%d)%s(%d)",CD->Field,CD->nt,CD->SubField,CD->ns);
 }
 
#ifndef _UNIX_
extern "C" {
#endif

char* LireCD_C( TypeCD* CD ) // En mode rule edit, demander le contenu du CD
 {
//   printf("Avant\n"); PrCD(CD);
   FinishCD(CD);
//   printf("\nApres\n"); PrCD(CD); printf("\n");

   TCDLib* aCDL=InputRecord->GetFirstCDLib();
   TCD     aCD(CD);
   char* ptr;

   if (InputRecord->NextCD(&aCDL,&aCD))
    ptr=strdup(aCDL->GetContent(&aCD));
   else
    ptr=strdup("");
   if (!ptr)
    ErrorHandler->SetErrorD(5000,ERROR,"While reading an input CD of a rule");
   return ptr;
 }
		
int Exists_C( TypeCD* CD ) // En mode rule edit, demander le contenu du CD (si ns ou nt a 0 => 1)
 {
   FinishTCD(CD);

   TCDLib* aCDL=InputRecord->GetFirstCDLib();
   TCD     aCD(CD);

   return InputRecord->NextCD(&aCDL,&aCD);
 }

int Precedes_C( TypeCD* CD1, TypeCD* CD2 ) // En mode rule edit, demander le champ complet contenant les 2 CD
 {
   FinishCD(CD1);
   FinishCD(CD2);

   TCDLib* aCDL=InputRecord->GetFirstCDLib();
   TCD     aCD1(CD1);
   TCD     aCD2(CD2);

   if (!InputRecord->NextCD(&aCDL,&aCD1)) return 0;
   if (!InputRecord->NextCD(&aCDL,&aCD2)) return 0;
   else return 1;
 }

char* NextSubField_C( TypeCD* CD1, TypeCD* CD2 ) // Idem
 {
   FinishCD(CD1);
   FinishCD(CD2);

   TCDLib* aCDL=InputCDL;
   TCD     aCD1(CD1);
   char *ptr=NULL;

   if (aCDL) aCDL=(TCDLib*)aCDL->GetNext();
   if (CD2==NULL)
    {
     if (InputRecord->NextCD(&aCDL,&aCD1)) 
      ptr=strdup(aCDL->GetContent());
    }
   else
    {
     TCD   aCD2(CD2);
     while(aCDL)
      {
       if (*aCDL==aCD1) { ptr=strdup(aCDL->GetContent()); break; }
       if (*aCDL==aCD2) { ptr=strdup(""); break; }
       aCDL=(TCDLib*)aCDL->GetNext();
      }
   }
  if (!ptr) ptr=strdup("");
  if (!ptr)
    ErrorHandler->SetErrorD(5000,ERROR,"While reading a next subfield");
  return ptr;
 }


char* LastSubField_C( TypeCD* CD1, TypeCD* CD2 ) // Idem
 {
   FinishCD(CD1);
   FinishCD(CD2);

   TCDLib* aCDL=InputCDL;
   TCD     aCD1(CD1);
   char *ptr=NULL;

   if (aCDL) aCDL=(TCDLib*)aCDL->GetPrevious();
   if (CD2==NULL)
    {
     if (InputRecord->PreviousCD(&aCDL,&aCD1)) 
      ptr=strdup(aCDL->GetContent());
    }
   else
    {
     TCD   aCD2(CD2);
     while(aCDL)
      {
       if (*aCDL==aCD1) { ptr=strdup(aCDL->GetContent()); break; }
       if (*aCDL==aCD2) { ptr=strdup(""); break; }
       aCDL=(TCDLib*)aCDL->GetPrevious();
      }
   }
  if (!ptr) ptr=strdup("");
  if (!ptr)
    ErrorHandler->SetErrorD(5000,ERROR,"While reading a previous subfield");
  return ptr;
 }









char* Table( char* Nom, char* str )
 {
  TRuleFile		*aRuleFile;
  TCodedData	*aCodedData;
  
  aRuleFile=RuleDoc->GetFile();
  aCodedData=aRuleFile->GetCodedData(Nom);
  if (!aCodedData)
   return strdup(str);  //ERREUR

  aCodedData->Transcode(str,(char*)Temporary,(InputRecord)?InputRecord->GetFirstField()->GetLib():NULL,"???");


  return strdup((char*)Temporary);
 }


void ResetSort()
 {
   if (ListSort) delete ListSort;
   ListSort=NULL;   
 }  

int MustSort( char* n )
 {
   if (ListSort)
    {
     LastSort->SetNext(new SortElem(theCDOut, n));
     LastSort=LastSort->GetNext();
    }
   else
    ListSort=LastSort=new SortElem(theCDOut, n);
  return 0;
 }

#ifndef _UNIX_
}
#endif


int SortRecord(TUMRecord* aRecord)
 {
  SortElem* aElem=ListSort;
  while( aElem )
   {
    aRecord->SortCD( aElem->GetCD(), aElem->GetSubFieldList() );
    aElem=aElem->GetNext();
   }
  ResetSort();
  return 0;
 }

//
// Renvoie la balise du CDLib suivant le CDLib courant, dans le champ courant
//
char* NextBalise()
 {
   TCDLib* aCDL=InputCDL;
   TCDLib* Suivant;
   char	   balise[3];

   Suivant=(TCDLib*)aCDL->GetNext();
   if (Suivant!=NULL)
   {
	   if (strcmp(aCDL->GetTag(),Suivant->GetTag())==0)
	   {
		   strcpy(balise,Suivant->GetSubfield());
		   return strdup(&balise[1]);
	   }
   }
   return NULL;
 }

//
// Renvoie la balise du CDLib prcdant le CDLib courant, dans le champ courant
//
char* PreviousBalise()
 {
   TCDLib* aCDL=InputCDL;
   TCDLib* Precedant;
   char	   balise[3];

   Precedant=(TCDLib*)aCDL->GetPrevious();
   if (Precedant!=NULL)
   {
	   if (strcmp(aCDL->GetTag(),Precedant->GetTag())==0)
	   {
		   strcpy(balise,Precedant->GetSubfield());
		   return strdup(&balise[1]);
	   }
   }
   return NULL;
 }



