/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  tchckdoc.cpp
 *
 *  implements a class to manage the checking of MARC records
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h>
#include "error.h"

#include "defines.h"
#include "tchckdoc.h"
#include "tmpplctn.h"
#include "xvtstuff.h"


extern TUMApplication *g_theApplication;




///////////////////////////////////////////////////////////////////////////////
//
// Document class constructor
//
// Create and initialize the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TCheckDoc::TCheckDoc( )
{
 itsInputFile		= NULL;
 itsOutputFile		= NULL;
 itsCheckInputSpec	= NULL;
 itsCheckOutputSpec	= NULL; 
 itsXVTFilePointer	= NULL; 
}

///////////////////////////////////////////////////////////////////////////////
//
// Document class destructor
//
// Delete the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TCheckDoc::~TCheckDoc( void )
{ 
 g_theApplication -> SetCheckDoc( NULL );
 if (itsInputFile)		{ delete itsInputFile;		itsInputFile		= NULL; }
 if (itsOutputFile)		{ delete itsOutputFile;		itsOutputFile		= NULL; }
 if (itsCheckInputSpec)	{ delete itsCheckInputSpec;	itsCheckInputSpec	= NULL; }
 if (itsCheckOutputSpec){ delete itsCheckOutputSpec;itsCheckOutputSpec	= NULL; }   

 // John Hough - added delete 
 if (itsXVTFilePointer)
	 delete itsXVTFilePointer;
}



///////////////////////////////////////////////////////////////////////////////
//
// Open
//
// Called by DoOpen().  Do the actual work of restoring the object from
// a data source.
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN TCheckDoc::Open(int IO)
{
  // Add code here to set internal data members of the class based
  // on information read from some data source 
 switch (IO)
  {
   case INPUT: 
    if (!itsInputFile)		// there is no Input Check File
     { 
      if ((itsInputFile = new TCheckFile(itsXVTFilePointer))==NULL)
       {
        ErrorHandler->SetError(9011,ERROR);
        return FALSE;
       } 
     } 
    if (itsInputFile->Open(IO))
     {
      delete itsInputFile;
      itsInputFile = NULL;
      return FALSE;
     }
    if (!itsCheckInputSpec)
     itsCheckInputSpec = new FILE_SPEC;
    memcpy(itsCheckInputSpec,itsXVTFilePointer,sizeof(FILE_SPEC));        
    break;
   case OUTPUT: 
    if (!itsOutputFile)		// there is no Output Check File
     { 
      if ((itsOutputFile = new TCheckFile(itsXVTFilePointer))==NULL)
       {
        ErrorHandler->SetError(9012,ERROR);
        return FALSE;
       }
     }  
    if (itsOutputFile->Open(IO))
     {
      delete itsOutputFile;
      itsOutputFile = NULL;
      return FALSE;
     }
    if (!itsCheckOutputSpec)
     itsCheckOutputSpec = new FILE_SPEC;
    memcpy(itsCheckOutputSpec,itsXVTFilePointer,sizeof(FILE_SPEC));        
    break;
   default:
    return FALSE;
   break;
  }
 return TRUE;   
}

 
   
///////////////////////////////////////////////////////////////////////////////
//
// OpenCheckFile
//
///////////////////////////////////////////////////////////////////////////////       
BOOLEAN TCheckDoc::OpenCheckFile(int IO,char *DefaultFile)
{
 int			Result;
 char			*ItemPointer; 
 
 if (!itsXVTFilePointer)
  itsXVTFilePointer = new FILE_SPEC;
 memset(itsXVTFilePointer,0,sizeof(FILE_SPEC)); 
 if (DefaultFile)
 // The Input/Output Marc Checking table has already been selectionned
  {     
   if ((ItemPointer=(char *)strrchr(DefaultFile,SLASH))!=NULL)
    {
     strcpy(itsXVTFilePointer->name,ItemPointer+1);      
     itsXVTFilePointer->type[0]=0;
     *ItemPointer = NULL;
     xvt_fsys_convert_str_to_dir(DefaultFile,itsXVTFilePointer->dir);
    }                                                           
   else
    { 
     strcpy(itsXVTFilePointer->name,DefaultFile);
     itsXVTFilePointer->type[0]=0;            
     xvt_fsys_get_dir(itsXVTFilePointer->dir);
    }
   Result=FL_OK;
  }
  
 switch(Result)  
  {
   case FL_OK:						// a Trans file has been selected
    if (Open(IO)!=TRUE)
     return FALSE;      
    return TRUE;
    break;
   case FL_CANCEL:
    return TRUE;
    break;
   case FL_BAD:
   default:
    return FALSE;
  }
}     
   
