/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  tmarcdoc.cpp
 *
 *  implements a pseudo-document class to manage MARC data
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h>
#include "error.h"
#include "defines.h"

#include "tmpplctn.h"
#include "tmarcdoc.h"
#include "dtcnfdlg.h"


extern TUMApplication *g_theApplication;


///////////////////////////////////////////////////////////////////////////////
//
// Document class constructor
//
// Create and initialize the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TMarcDoc::TMarcDoc( ) 
{
 itsEditConfDlg                 = NULL; 
 itsInputFile                   = NULL;
 itsOutputFile                  = NULL;
 itsMarcInputSpec               = NULL;
 itsMarcOutputSpec              = NULL;
 itsInputRecord                 = NULL;
 itsTransRecord                 = NULL;
 itsOutputRecord                = NULL;
 itsConfInputSpec               = NULL;
 itsConfOutputSpec              = NULL;
 itsFirstInputTagNoInd  = NULL;
 itsFirstOutputTagNoInd = NULL; 
 itsMarcFile.Spec               = NULL;
 itsMarcFile.Conf               = NULL;
 itsMarcFile.Format				= FALSE;
 itsMarcFile.BlockSize  = 2048;
 itsMarcFile.MinDataFree= 5;
 itsMarcFile.PaddingChar= 0x5E;
 itsMarcFile.LastBlock  = FALSE;
 itsXVTFilePointer	= NULL; 
}

///////////////////////////////////////////////////////////////////////////////
//
// Document class destructor
//
// Delete the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TMarcDoc::~TMarcDoc( void )
{ 
 g_theApplication -> SetMarcDoc( NULL );
 if (itsInputFile)                      { delete itsInputFile;                  itsInputFile                    = NULL; }
 if (itsMarcInputSpec)          { delete itsMarcInputSpec;              itsMarcInputSpec                = NULL; } 
 if (itsMarcOutputSpec)         { delete itsMarcOutputSpec;             itsMarcOutputSpec               = NULL; } 
 if (itsOutputFile)                     { delete itsOutputFile;                 itsOutputFile                   = NULL; }
 if (itsInputRecord)            { delete itsInputRecord;                itsInputRecord                  = NULL; }  
 if (itsTransRecord)            { delete itsTransRecord;                itsTransRecord                  = NULL; }  
 if (itsOutputRecord)           { delete itsOutputRecord;               itsOutputRecord                 = NULL; }   
 if (itsMarcFile.Spec)          { delete itsMarcFile.Spec;              itsMarcFile.Spec                = NULL; }  
 if (itsMarcFile.Conf)          { delete itsMarcFile.Conf;              itsMarcFile.Conf                = NULL; }
 if (itsConfInputSpec)          { delete itsConfInputSpec;              itsConfInputSpec                = NULL; } 
 if (itsConfOutputSpec)         { delete itsConfOutputSpec;             itsConfOutputSpec               = NULL; } 
 DelTreeTagNoInd(itsFirstInputTagNoInd);
 DelTreeTagNoInd(itsFirstOutputTagNoInd); 

 

}


///////////////////////////////////////////////////////////////////////////////
//
// Open
//
// Called by DoOpen().  Do the actual work of restoring the object from
// a data source.
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN TMarcDoc::Open(int IO)
{

	// John Hough
	char *marcInput = "00714nas  2200217   45  001000900000008004100009015001300050022001300063082001600076222001900092245006000111255002400171260010700195300002200302350001300324554001400337650003900351650004000390650003800430650002800468.b9972127.991015m19989999en a    W    00001  engpb.00.ab9972127.00.a13623133.00.a332.042.c21.00.aGlobal trading.30.aGlobal trading.bincorporating Universal Risk Management.00.a3rd quarter (1998)-.00.aLondon.dAlexander Russell House, 50 Great Eastern St., London EC2A 3EP.bGlobal Trading Magazine.c1998-.00.nv..bcol.ill.c30cm.00.aNo price.00.aQuarterly.00.aInternational finance.vPeriodicals.00.aFinancial institutions.vPeriodicals.00.aInvestments, Foreign.vPeriodicals.00.aSecurities.vPeriodicals..00823nas  2200229   45  001000900000008004100009015001300050022001300063082001600076222003300092245003200125255001500157260009700172300004000269350001300309554001400322650004200336650002300378710002900401910006300430910010000493.b9972134.991015m19999999en ac   W    00001  engpb.00.ab9972134.00.a14670526.00.a636.234.c21.00.aHolstein news & sire reviews.30.aHolstein news & sire review.00.aSept. '99-.00.aRickmansworth.dScotsbridge House, Rickmansworth, Herts WD3 3BB.bHolstein UK & Ireland.c1999-.00.nv..bill(some col.).icol.ports.c30cm.00.aNo price.00.aQuarterly.00.aHolstein-Friesian cattle.vPeriodicals.00.aBulls.vPeriodicals.21.aHolstein, UK and Ireland.20.aHolstein, UK & Ireland.xSee.aHolstein, UK and Ireland.z710.20.aHolstein Friesian Society of Great Britain and Ireland.xSee also.aHolstein, UK and Ireland.z710..";


 // Add code here to set internal data members of the class based
 // on information read from some data source
 switch (IO)
  {
   case INPUT: 
    if (!itsInputFile)          // there is no Marc Input File
     {   
      if ((itsInputFile = new TMarcFile(itsMarcFile.Spec,FILE_READ,FILE_BINARY))==NULL)
       {
        ErrorHandler->SetError(9021,ERROR);
        return FALSE;       
       }
     }    
    if (itsInputFile->Open())
     {
      delete itsInputFile;
      itsInputFile = NULL;
      return FALSE;
     } 



    if ((itsInputRecord = new TUMRecord())==NULL)
     {
      ErrorHandler->SetError(9031,ERROR);
      return FALSE;
     }


	// John Hough - direct string interface
	//itsInputFile->SetBuffer(marcInput);


    if (!itsMarcInputSpec)
     itsMarcInputSpec = new FILE_SPEC;
    memcpy(itsMarcInputSpec,itsMarcFile.Spec,sizeof(FILE_SPEC));
    
    if (itsMarcFile.Conf)
     { 
      g_theApplication ->convert_file_spec_to_str(itsMarcFile.Conf,(char *)Temporary);
      if (GetFirstTagNoInd(INPUT))
       UnloadTagNoInd(INPUT);
      if (!LoadTagNoInd(INPUT,(char *)Temporary))
       {
        if (!itsConfInputSpec)
         itsConfInputSpec = new FILE_SPEC;
        memcpy(itsConfInputSpec,itsMarcFile.Conf,sizeof(FILE_SPEC));
       }
      if (itsMarcFile.Conf)             { delete itsMarcFile.Conf;              itsMarcFile.Conf                = NULL; }  
     }
    else
     {
      if (GetFirstTagNoInd(INPUT))
       UnloadTagNoInd(INPUT);
      if (itsConfInputSpec)     { delete itsConfInputSpec;      itsConfInputSpec        = NULL; }
     }  
    break;
    
   case OUTPUT: 
    if (!itsOutputFile)         // there is no Marc Output File
     { 
      if ((itsOutputFile = new TMarcFile(/*(CDocument *)this,*/itsMarcFile.Spec,FILE_WRITE,FILE_BINARY))==NULL)
       {
        ErrorHandler->SetError(9022,ERROR);
        return FALSE;
       }
     }    
    if (itsOutputFile->Open())
     {
      delete itsOutputFile;
      itsOutputFile = NULL;
      return FALSE;
     }
             
    if ((itsOutputRecord = new TUMRecord())==NULL)
     {
      ErrorHandler->SetError(9032,ERROR);
      return FALSE;
     }
    if (!itsMarcOutputSpec)
     itsMarcOutputSpec = new FILE_SPEC;
    memcpy(itsMarcOutputSpec,itsMarcFile.Spec,sizeof(FILE_SPEC)); 

    if (itsMarcFile.Conf)
     { 
      g_theApplication->convert_file_spec_to_str(itsMarcFile.Conf,(char *)Temporary);
      if (GetFirstTagNoInd(OUTPUT))
       UnloadTagNoInd(OUTPUT);
      if (!LoadTagNoInd(OUTPUT,(char *)Temporary))
       {
        if (!itsConfOutputSpec)
         itsConfOutputSpec = new FILE_SPEC;
        memcpy(itsConfOutputSpec,itsMarcFile.Conf,sizeof(FILE_SPEC));
       }
      if (itsMarcFile.Conf)             { delete itsMarcFile.Conf;              itsMarcFile.Conf                = NULL; }  
     }
    else 
     {
      if (GetFirstTagNoInd(OUTPUT))
       UnloadTagNoInd(OUTPUT);
      if (itsConfOutputSpec)    { delete itsConfOutputSpec;     itsConfOutputSpec       = NULL; }
     }        
    break;
    
   default:
    return FALSE;
   break;
  }
 return TRUE;
} 



// MPB - 16-03-99 : pour StartBatch()
BOOLEAN TMarcDoc::Close(int IO)  
{ 
 switch(IO)
  {
   case INPUT:
    if (itsInputFile)                                           // a Marc Input File is already loaded
     { 
        // Current Marc Input File closing is confirmed
         if (itsInputFile->Close())                             // Close the current Marc Input File
          return FALSE;                                         // Current Marc Input Closing failure
         delete itsInputFile; 
         itsInputFile = NULL;
         if (itsMarcInputSpec)  { delete itsMarcInputSpec;      itsMarcInputSpec=NULL;  }
     } 
    break;
   case OUTPUT: 
    if (itsOutputFile)                                          // a Marc Output File is already loaded
     { 
        if (itsOutputFile->Close())             // Close the current Marc Output File
          return FALSE;
        delete itsOutputFile;                           // Current Marc Output Closing failure
        itsOutputFile = NULL;
        if (itsMarcOutputSpec)  { delete itsMarcOutputSpec;     itsMarcOutputSpec=NULL; }
     }  
    break;
   default:
    return FALSE;
    break;      
  }  
 return TRUE;                                               
}  



///////////////////////////////////////////////////////////////////////////////
//
// Convert
//
///////////////////////////////////////////////////////////////////////////////
int TMarcDoc::Convert(TRuleDoc *RuleDoc)
{   
 // On reset le EH
 ErrorHandler->Reset();

 // On commence par convertir itsTransRecord sous la forme utilisable par la 
 // conversion de regles, a savoir une suite de CDLib. 
 itsTransRecord->ToCD();

 // Si une notice de sortie existe, on la vide
 itsOutputRecord->DelTreeCDLib();
 itsOutputRecord->DelTree();

 // On convertit alors la notice itsTransRecord en itsOutputRecord conformement
 // au fichier de regles
 if (RuleDoc->GetFile())
  RuleDoc->GetFile()->Convert(itsTransRecord,itsOutputRecord);
 else
  { 
   if (itsOutputRecord) { delete itsOutputRecord; itsOutputRecord=NULL; }
   itsOutputRecord = new TUMRecord(itsTransRecord);  
   if (!itsOutputRecord)
    ErrorHandler->SetErrorD(3000,ERROR,"When copying Input record to Output record" );
  }

 // On reconstruit alors la notice de sortie sous la forme d'une liste de champs,
 // a partir de la liste des CDLib.
 itsOutputRecord->FromCD();

 return ErrorHandler->GetErrorCode();
}

///////////////////////////////////////////////////////////////////////////////
//
// Transcode
//
///////////////////////////////////////////////////////////////////////////////
int TMarcDoc::Transcode(TTransDoc *aTransDoc)
 {
  TCheckDoc             *aCheckDoc;
      
  // On reset le EH
  ErrorHandler->Reset();      
      
  // Check input
  aCheckDoc= g_theApplication->GetCheckDoc();
  if (aCheckDoc)
   if (aCheckDoc->GetInputFile())
    aCheckDoc->GetInputFile()->Verify(INPUT,itsInputRecord);
 
  // On reset le EH
  ErrorHandler->Reset();

  // Si itsTransRecord existe, on la supprime
  if (itsTransRecord) delete itsTransRecord;
  // Si aucun fichier de transco n'a ete charge, itsTransRecord est une simple copie
  // de la notice d'entree.
  // Si un fichier existe, alors on convertit itsInputRecord en itsTransRecord conformement
  // a la table de transco chargee

  if (aTransDoc->GetFile()==NULL)
   {
    itsTransRecord=new TUMRecord( itsInputRecord );
    if (!itsTransRecord)
     ErrorHandler->SetErrorD( 3000, ERROR, "When copying Input record to Trans record" );
   }
  else
   {
    itsTransRecord=new TUMRecord;
    if (!itsTransRecord)
     ErrorHandler->SetErrorD( 3000, ERROR, "When creating a new Trans record" );
    aTransDoc->GetFile()->Convert( itsInputRecord, itsTransRecord );
   }

  return ErrorHandler->GetErrorCode();
 }
          
///////////////////////////////////////////////////////////////////////////////
//
// DelTreeTagNoInd
//
///////////////////////////////////////////////////////////////////////////////
void TMarcDoc::DelTreeTagNoInd(TTagNoInd *Start)
{
 TTagNoInd      *Courant,
                        *Suivant;
                        
 Courant=Start;
 while (Courant)
  { 
   Suivant = Courant->GetNext();
   delete Courant;
   Courant = Suivant;
  }
 Start = NULL;                          
}          
 
///////////////////////////////////////////////////////////////////////////////
//
// SetMarcFileSpec
//
///////////////////////////////////////////////////////////////////////////////
void TMarcDoc::SetMarcFileSpec(FILE_SPEC *theSpec)
{ 
 if (itsMarcFile.Spec==NULL) 
  itsMarcFile.Spec=new FILE_SPEC;
 memcpy(itsMarcFile.Spec,theSpec,sizeof(FILE_SPEC)); 
} 
          
///////////////////////////////////////////////////////////////////////////////
//
// SetConfFileSpec
//
///////////////////////////////////////////////////////////////////////////////
void TMarcDoc::SetConfFileSpec(FILE_SPEC *theSpec)
{
 if (!itsMarcFile.Conf) 
 itsMarcFile.Conf=new FILE_SPEC;
 memcpy(itsMarcFile.Conf,theSpec,sizeof(FILE_SPEC)); 
}
 
///////////////////////////////////////////////////////////////////////////////
//
// SetFirstTagNoInd
//
///////////////////////////////////////////////////////////////////////////////
void TMarcDoc::SetFirstTagNoInd(int IO,TTagNoInd *aTagNoInd)
{
 switch(IO)
  {
   case INPUT   : itsFirstInputTagNoInd = aTagNoInd;    break;
   case OUTPUT  : itsFirstOutputTagNoInd= aTagNoInd;    break;
   default              : break;
  } 
} 
  
///////////////////////////////////////////////////////////////////////////////
//
// GetFirstTagNoInd
//
///////////////////////////////////////////////////////////////////////////////
TTagNoInd *TMarcDoc::GetFirstTagNoInd(int IO)
{    
 switch(IO)
  {
   case INPUT   : return itsFirstInputTagNoInd;
   case OUTPUT  : return itsFirstOutputTagNoInd;
   default              : return NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
//
// UnloadTagNoInd
//
///////////////////////////////////////////////////////////////////////////////
int TMarcDoc::UnloadTagNoInd(int IO)
{
 TTagNoInd      *aTagNoInd;
 
 switch (IO)
  {
   case INPUT   : 
    aTagNoInd = itsFirstInputTagNoInd;
    itsInputRecord->SetFirstInputTNI(NULL);
    break;  
    
   case OUTPUT  :
    aTagNoInd = itsFirstOutputTagNoInd;
    itsOutputRecord->SetFirstOutputTNI(NULL);
    break;
    
   default              : 
    aTagNoInd = NULL;
    break;
  }
 DelTreeTagNoInd(aTagNoInd);
 return 0;  
}   

///////////////////////////////////////////////////////////////////////////////
//
// LoadTagNoInd
//
///////////////////////////////////////////////////////////////////////////////
int TMarcDoc::LoadTagNoInd(int IO, char *Path)
{ 
 FILE           *Fic;
 TTagNoInd      *Current,
                        *Previous;
    
 if ((Fic=fopen(Path,"r"))==NULL)
  return 1;
  
 Current = new TTagNoInd;
 if (!Current)
  return 1;
 switch (IO)
  {
   case INPUT   : itsFirstInputTagNoInd = Current;      break;
   case OUTPUT  : itsFirstOutputTagNoInd= Current;      break;
   default              : return 1;
  }
 Previous = Current;
 while (fgets((char *)Temporary,BUFF_SIZE,Fic))
  {
   Current->SetTag((char *)Temporary);
   Current->SetNext(new TTagNoInd);
   if (!Current->GetNext())
    return 1;
   Previous = Current;
   Current = Current->GetNext(); 
  }
 if (Previous->GetNext())                              
  delete Previous->GetNext();
 Previous->SetNext(NULL);

 fclose(Fic);
 
 switch (IO)
  {
   case INPUT   : itsInputRecord->SetFirstInputTNI(itsFirstInputTagNoInd);              break;
   case OUTPUT  : itsOutputRecord->SetFirstOutputTNI(itsFirstOutputTagNoInd);   break;
   default              : return 1;
  } 
 return 0;
} 

int TMarcDoc::LoadTagNoInd(int IO)
{ 
 TTagNoInd      *Current,
                        *Previous;
 int            Indice,
                        NbOfLoop;
 
 if (!itsEditConfDlg)     
  return 1; 
  
 NbOfLoop=itsEditConfDlg->GetNbTagsNoInd();
 if (!NbOfLoop)
  switch (IO)
  {
   case INPUT   : itsFirstInputTagNoInd = NULL; return 0;
   case OUTPUT  : itsFirstOutputTagNoInd= NULL; return 0;
   default      : return 1;
  }
  
 Current = new TTagNoInd;
 if (!Current)
  return 1;
 switch (IO)
  {
   case INPUT   : itsFirstInputTagNoInd = Current;      break;
   case OUTPUT  : itsFirstOutputTagNoInd= Current;      break;
   default              : return 1;
  }
 Previous = Current;
 for  (Indice=0; Indice<NbOfLoop; Indice++)
  { 
//   Current->SetTag((itsEditConfDlg->GetCurrentTagNoInd(Indice)).data());
   Current->SetNext(new TTagNoInd);
   if (!Current->GetNext())
    return 1;
   Previous = Current;
   Current = Current->GetNext(); 
  }
 if (Previous->GetNext())                              
  delete Previous->GetNext();
 Previous->SetNext(NULL);
 
 switch (IO)
  {
   case INPUT   : itsInputRecord->SetFirstInputTNI(itsFirstInputTagNoInd);              break;
   case OUTPUT  : itsOutputRecord->SetFirstOutputTNI(itsFirstOutputTagNoInd);   break;
   default              : return 1;
  } 
 
 return 0;
}  
  
///////////////////////////////////////////////////////////////////////////////
//
// SaveTagNoInd
//
///////////////////////////////////////////////////////////////////////////////
int TMarcDoc::SaveTagNoInd(int IO, char *Path)
{ 
 FILE           *Fic;
 TTagNoInd      *Current;
    
 if ((Fic=fopen(Path,"w"))==NULL)
  return 1;
  
 switch (IO)
  {
   case INPUT   : Current = itsFirstInputTagNoInd;      break;
   case OUTPUT  : Current = itsFirstOutputTagNoInd;     break;
   default              : Current = NULL;               break;
  } 
 while (Current)
  {
   fprintf(Fic,"%s\n",Current->GetTag());
   Current = Current->GetNext(); 
  }
 
 fclose(Fic);
 return 0;
} 

///////////////////////////////////////////////////////////////////////////////
//
// SetIndSeparatorsID
//
///////////////////////////////////////////////////////////////////////////////
void TMarcDoc::SetIndSeparatorsID(int IO, long ID)
{ 
 switch (IO)
  {
   case INPUT   : itsInputRecord->SetIndSeparatorsID(IO,ID);    break;
   case OUTPUT  : itsOutputRecord->SetIndSeparatorsID(IO,ID);   break;
   default              : break;
  }
}
 
///////////////////////////////////////////////////////////////////////////////
//
// GetIndSeparatorsID
//
///////////////////////////////////////////////////////////////////////////////
long TMarcDoc::GetIndSeparatorsID(int IO)
{ 
 switch (IO)
  {
   case INPUT   : return (itsInputRecord ? itsInputRecord->GetIndSeparatorsID(IO) : 0);
   case OUTPUT  : return (itsOutputRecord ? itsOutputRecord->GetIndSeparatorsID(IO) : 0);
   default              : return 0;
  }
} 


















