/* 
 *  USEMARCON Software - Command Line version
 *  
 *  File:  chckfile.h
 *
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *

CLASS
    TMarcDoc

	implements a class to manage MARC files

OVERVIEW TEXT
    USEMARCON Software - Command Line version
    Copyright The British Library, The USEMarcon Consortium, 1995-2000
    Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
    
AUTHOR
    Crossnet Systems Limited

*/

#ifndef TMarcDoc_H
#define TMarcDoc_H

#include "marcfile.h"
#include "umrecord.h"
#include "tagnoind.h"
#include "defines.h"
#include "ttrnsdoc.h"
#include "dtcnfdlg.h"

#define STATUS_OK               1                        
#define SEGMENTED               1
#define LAST_PADDED             1


class TMarcDoc 
{
public:

//////////
							TMarcDoc   ();
//////////
  virtual                   ~TMarcDoc  (void);
  
//////////
  virtual BOOLEAN Open                 (int IO);
//////////
  virtual BOOLEAN Close                (int IO);       
                                       
                                       
//////////
  virtual int     Convert              (TRuleDoc *aRuleDoc);
//////////
  virtual int     Transcode            (TTransDoc *aTransDoc);
//////////
  virtual int     LoadTagNoInd         (int IO,char *Path);
//////////
  virtual int     LoadTagNoInd         (int IO);
//////////
  virtual int     SaveTagNoInd         (int IO,char *Path);
//////////
  virtual int     UnloadTagNoInd       (int IO);
  
//////////
  virtual void    DelTreeTagNoInd      (TTagNoInd *Start);     
//////////
  virtual void    SetMarcFileSpec      (FILE_SPEC *theSpec );  
//////////
  virtual void    SetMarcFileFormat    (BOOLEAN theFormat)    
{ itsMarcFile.Format=theFormat; };  
//////////
  virtual void    SetMarcFileBlockSize    (short theBlockSize)    {
itsMarcFile.BlockSize=theBlockSize; };  
//////////
  virtual void    SetMarcFileMinDataFree  (short theMinDataFree)  {
itsMarcFile.MinDataFree=theMinDataFree; };  
//////////
  virtual void    SetMarcFilePaddingChar  (char thePaddingChar)   {
itsMarcFile.PaddingChar=thePaddingChar; };  
//////////
  virtual void    SetMarcFileLastBlock    (BOOLEAN theLastBlock)  {
itsMarcFile.LastBlock=theLastBlock; };
//////////
  virtual void    SetConfFileSpec      (FILE_SPEC *theSpec); 
//////////
  virtual void    SetFirstTagNoInd     (int IO,TTagNoInd *aTagNoInd);
//////////
  virtual void          SetIndSeparatorsID              (int IO,long ID);
                                                    
//////////
  virtual TMarcFile     *GetInputFile     (void)                  
{ return itsInputFile; };
//////////
  virtual TMarcFile     *GetOutputFile    (void)                  
{ return itsOutputFile; };
//////////
  virtual TUMRecord     *GetInputRecord   (void)                  
{ return itsInputRecord; }; 
//////////
  virtual TUMRecord     *GetOutputRecord  (void)                  
{ return itsOutputRecord; }; 
//////////
  virtual TUMRecord     *GetTransRecord   (void)                  
{ return itsTransRecord; };                                              
     
//////////
  virtual long          GetIndSeparatorsID  (int IO);
//////////
  virtual FILE_SPEC     *GetMarcFileSpec    (void)                 
{ return itsMarcFile.Spec; };
//////////
  virtual BOOLEAN       GetMarcFileFormat   (void)                 
{ return itsMarcFile.Format; };  
//////////
  virtual short         GetMarcFileBlockSize    (void)
{ return itsMarcFile.BlockSize; };  
//////////
  virtual short         GetMarcFileMinDataFree  (void)
{ return itsMarcFile.MinDataFree; };  
//////////
  virtual char          GetMarcFilePaddingChar  (void)                  
{ return itsMarcFile.PaddingChar; }; 
//////////
  virtual BOOLEAN       GetMarcFileLastBlock    (void)                  
{ return itsMarcFile.LastBlock; };  
//////////
  virtual FILE_SPEC     *GetMarcInputSpec       (void)                 
{ return itsMarcInputSpec; };
//////////
  virtual FILE_SPEC     *GetMarcOutputSpec      (void)                 
{ return itsMarcOutputSpec; };
//////////
  virtual FILE_SPEC     *GetConfInputSpec       (void)                 
{ return itsConfInputSpec; }
//////////
  virtual FILE_SPEC     *GetConfOutputSpec      (void)                 
{ return itsConfOutputSpec; }  
//////////
  virtual FILE_SPEC     *GetMarcConfSpec        (void)                 
{ return itsMarcFile.Conf; };
//////////
  virtual TTagNoInd     *GetFirstTagNoInd       (int IO);
//////////
  virtual long          GetInputRecordNumber    (void)
{ return itsInputFile->GetRecordNumber(); };  
//////////
  virtual long          GetOutputRecordNumber   (void)
{ return itsOutputFile->GetRecordNumber(); };
  
//////////
  virtual int           Read                    (void)          
{ return itsInputFile->Read(itsInputRecord); };
//////////
  virtual int           Write                   (void)          
{ return itsOutputFile->Write(itsOutputRecord); };

                                                                   
private:

//////////
  TEditConfDlg                  *itsEditConfDlg; 
//////////
  TUMRecord                     *itsInputRecord;
//////////
  TUMRecord                     *itsTransRecord;
//////////
  TUMRecord                     *itsOutputRecord;          
//////////
  TMarcFile                     *itsInputFile;
//////////
  TMarcFile                     *itsOutputFile;
//////////
  FILE_SPEC                     *itsMarcInputSpec;
//////////
  FILE_SPEC                     *itsMarcOutputSpec;
//////////
  FILE_SPEC                     *itsConfInputSpec;
//////////
  FILE_SPEC                     *itsConfOutputSpec;
//////////
  FILE_SPEC                     *itsXVTFilePointer;
//////////
  TTagNoInd                     *itsFirstInputTagNoInd;
//////////
  TTagNoInd                     *itsFirstOutputTagNoInd;
//////////
  struct MARCFILE
   {
    FILE_SPEC                   *Spec;
    FILE_SPEC                   *Conf;
    BOOLEAN                     Format;
    short                       BlockSize;
    short                       MinDataFree;
    char                        PaddingChar;
    BOOLEAN                     LastBlock;
   }itsMarcFile; 

};

#endif
