/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  tmpplctn.cpp
 *
 *  implements the main operations of the software
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 *  Rev history:
 *  17 Aug 2000 -  version raised to v1.1 - will not exit if the input or output
	checking files are not present in the .ini file.

 */

#define _MAIN_
#include <direct.h>
#include <stdlib.h>
#include <stdio.h>
#include <afx.h>
#include "xvtstuff.h"

#include "error.h"
#include "tools.h"

#include "defines.h"
#include "tmpplctn.h"
      
#include "truledoc.h"
                  
#include "dtcnfdlg.h"
       
#include "objectlist.h"

extern int              itsArgc;
extern char             **itsArgv;
extern "C"
{
	int                     debug_rule;
	unsigned long   ordinal;
}

int debug_umrecord;
int ToHexa(const char* str);

// John Hough addition
extern TypeInst *g_D;

extern "C" {
extern int FreeMem( TypeInst* t );
};

///////////////////////////////////////////////////////////////////////////////
//
// Application class constructor
//
// Defines the initial state of the application object.
//
///////////////////////////////////////////////////////////////////////////////
TUMApplication::TUMApplication(void) /*: CApplication()*/
{


	
 itsRuleDoc = new TRuleDoc(); 
 itsTransDoc = new TTransDoc(); 
 itsCheckDoc = new TCheckDoc(); 
 itsMarcDoc = new TMarcDoc(); 
 



   ErrorHandler           = NULL;
 itsExecutionMode       = MODE_UNDEFINED;
 itsMarcInputNumero = 0L;
  _getcwd(itsStartupDir, 149);
  strcat(itsStartupDir, "\\");

}

///////////////////////////////////////////////////////////////////////////////
//
// Application class destructor
//
// Delete the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TUMApplication::~TUMApplication( void )
{
		// John Hough - added test and delete calls
		if (itsRuleDoc)
			delete itsRuleDoc;

		if (itsTransDoc)
			delete itsTransDoc; 

  
		if (itsCheckDoc)
			delete itsCheckDoc;
	
		if (itsMarcDoc)
			delete itsMarcDoc;
	

	
		if (ErrorHandler)   
		{ 
			delete ErrorHandler;
			ErrorHandler    = NULL; 
		} 
	
}
                           
///////////////////////////////////////////////////////////////////////////////
//
// Startup
//
// Defines the startup behavior for the application - whether an
// initial about box is created, or a login screen is presented,
// or an initial document object and its associated windows are
// created, etc.  Also, initialize application level objects here.
//
///////////////////////////////////////////////////////////////////////////////
void TUMApplication::StartUp(void)
{

	LoadIniFileDetails();

}



// John Hough - original Startup() function renamed
void TUMApplication::LoadIniFileDetails()
{
	// Get the first argument ( path to the initialisation file )

	// John Hough - added if
	if (myrun.GetReadMarcRecordsFromBexFile())
		printf("Loading .ini file parameters...\n");
	if (itsArgc>1)
	{
		
		strcpy(itsIniFile,itsArgv[1]);
		
		debug_rule=0;
		debug_umrecord=0; 
	}
	else
	{
		printf("Error: USAGE : usemarco xxxx.INI\nThe INI file is mandatory.\nYou must restart the application with a INI file specified");
		*itsIniFile=0;     
		debug_rule=0;
	}
	
	// Create an instance of TError 

	char           IniItem[BUFF_SIZE];

	
	
	GetPrivateProfileString("DEFAULT_FILES","ErrorLogFile","",IniItem,BUFF_SIZE,itsIniFile);
	if (*IniItem) {
		ErrorHandler  = new TError(this, IniItem);
	} else {
		ErrorHandler  = new TError(this);  
	}
	

	ErrorHandler->SetMode(INTERACTIVE);
	
	GetPrivateProfileString("DEFAULT_STATES","IsVerboseExecutionModeChecked","",IniItem,BUFF_SIZE,itsIniFile);
	if (!strcmp(IniItem,"TRUE")) {
		ErrorHandler->SetVerboseMode(1);
	} else {
		ErrorHandler->SetVerboseMode(0);
	}
	
	// Set the maximum number to be encoutered before stop  
	GetPrivateProfileString("DEFAULT_VALUES","MaxErrorsToBeEncoutered","",IniItem,BUFF_SIZE,itsIniFile);
	if (*IniItem) {
		ErrorHandler->SetTooManyErrors(atoi(IniItem));
	}                
	GetPrivateProfileString("DEFAULT_VALUES","OrdinalNumber","1",(char *)Temporary,BUFF_SIZE,itsIniFile);
	ordinal=atol((char *)Temporary);                   
	

		
	StartBatch();
	Convert();
	End();
}


// John Hough
double g_size;
BOOLEAN g_ConvOk=TRUE;
long g_NbTrait=0L;
FILE_SPEC *g_aFileSpec;

///////////////////////////////////////////////////////////////////////////////
//
// StartBatch
//
// Defines the startbatch behavior for the application - whether an
// initial about box is created, or a login screen is presented,
// or an initial document object and its associated windows are
// created, etc.  Also, initialize application level objects here.
//
///////////////////////////////////////////////////////////////////////////////
void TUMApplication::StartBatch(void)
{ 
	double	size;
	int	readcentile;
	int	oldreadcentile;

	if (!*itsIniFile)
    {
		ErrorHandler->WriteError("\nEND non OK : no FILE .ini");
		exit(-1);
    }
	
	if (!itsRuleDoc)
    {
		ErrorHandler->WriteError("\nEND non OK : No Rule structure created");
		exit(-1);
    }
	

	GetPrivateProfileString("DEFAULT_FILES","RuleFile","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (!*Temporary)
    {
		ErrorHandler->WriteError("\nEND non OK : No rulefile specified");
    }
	else
	{       
		if (itsRuleDoc->OpenRuleFile((char *)Temporary) == FALSE)
		{ 
		ErrorHandler->WriteError("\nEND non OK : Unable to open rulefile");
		}
	}
	
	if (!itsTransDoc)
    {
		ErrorHandler->WriteError("\nEND non OK : No transcoding structure created");
	}
	
	GetPrivateProfileString("DEFAULT_FILES","TranscodingCharacterTable","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (!*Temporary)
    {
		ErrorHandler->WriteError("\nEND non OK : No TranscodingCharacterTable specified");
    }
	else
    {
		itsTransDoc->OpenTransFile((char *)Temporary);
    }
	
	if (!itsCheckDoc)
    {
		ErrorHandler->WriteError("\nEND non OK : No transcoding structure created");
		exit(-1);
	}
	
	GetPrivateProfileString("DEFAULT_FILES","InputFormatCheckingTable","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (!*Temporary)
    {
		ErrorHandler->WriteError("\nNo input format checking table specified in the .ini file");
    }
	else
	{
		itsCheckDoc->OpenCheckFile(INPUT,(char *)Temporary);
    }
	
	if (!itsCheckDoc)
    {
		ErrorHandler->WriteError("\nEND non OK : No control structure created");
		exit(-1);
    }
	
	GetPrivateProfileString("DEFAULT_FILES","OutputFormatCheckingTable","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (!*Temporary)
    {
		ErrorHandler->WriteError("\nNo output format checking table specified in the .ini file");
    }
	else
    {
		itsCheckDoc->OpenCheckFile(OUTPUT,(char *)Temporary);
	}
	
	if (!itsMarcDoc)
    {
		ErrorHandler->WriteError("\nEND non OK : No marc structure created");
		exit(-1);
    }


	// rev 1.1 
	GetPrivateProfileString("DEFAULT_FILES","MarcInputFile","",(char *)Temporary,BUFF_SIZE,itsIniFile);


	if (!*Temporary )
    {
		ErrorHandler->WriteError("\nEND non OK : Input MARC file not specified in .ini file");
		exit(-1);
    }


	GetPrivateProfileString("DEFAULT_FILES","InputMarcEditConfigurationFile","",(char *)Temporary2,BUFF_SIZE,itsIniFile);
	if (!*Temporary2)
    {
		ErrorHandler->WriteError("\nNo Input MARC edit configuration file specified in the .ini file");
    }
	// end rev 1.1



	FILE_SPEC *aFileSpec = NULL;
	aFileSpec = new FILE_SPEC;
	memset(aFileSpec,0,sizeof(FILE_SPEC));
	strcpy(aFileSpec->name,(char *)Temporary);
	xvt_fsys_get_dir(aFileSpec->dir); 
	
	itsMarcDoc->SetMarcFileSpec(aFileSpec);
	
	itsMarcDoc->SetMarcFileFormat(FALSE);//non segmente par defaut
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","IsInputBlockSegmentedChecked","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary) 
	{
		if (!strcmp((char *)Temporary,"TRUE"))
		{
			itsMarcDoc->SetMarcFileFormat(TRUE);
		}
	}

	itsMarcDoc->SetMarcFileBlockSize(2048);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","InputMarcSizeBlock","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary) 
	{
		if (strcmp((char *)Temporary,"2048"))
		{
			itsMarcDoc->SetMarcFileBlockSize((short)atoi((char *)Temporary));
		}
	}

	itsMarcDoc->SetMarcFileMinDataFree(5);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","InputMarcMinDataFree","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary)
	{
		if (strcmp((char *)Temporary,"5"))
		{
			itsMarcDoc->SetMarcFileMinDataFree((short)atoi((char *)Temporary));
		}
	}

	itsMarcDoc->SetMarcFilePaddingChar(0x5E);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","InputMarcPaddingChar","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary) 
	{
		if (strcmp((char *)Temporary,"5E"))
		{
			itsMarcDoc->SetMarcFilePaddingChar((char)ToHexa((char *)Temporary));
		}
	}
	
	itsMarcDoc->SetMarcFileLastBlock(FALSE);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","IsInputLastBlockPaddedChecked","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary) 
	{
		if (!strcmp((char *)Temporary,"TRUE"))
		{
			itsMarcDoc->SetMarcFileLastBlock(TRUE);
		}
	}
	
	// John Hough - added delete
	if (aFileSpec)
		delete aFileSpec;

	aFileSpec = new FILE_SPEC;
	memset(aFileSpec,0,sizeof(FILE_SPEC));
	strcpy(aFileSpec->name,(char *)Temporary2);
	xvt_fsys_get_dir(aFileSpec->dir); 
	itsMarcDoc->SetConfFileSpec(aFileSpec);         
	
	itsMarcDoc->Open(INPUT);
	
	if (!itsMarcDoc)
	{
		ErrorHandler->WriteError("\nEND non OK : No marc structure created");
		exit(-1);
	}
		
	// ref 1.1			
	GetPrivateProfileString("DEFAULT_FILES","MarcOutputFile","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (!*Temporary)
	{
		ErrorHandler->WriteError("\nEND non OK : Output MARC file not specified in .ini file");
		exit(-1);
	}

	GetPrivateProfileString("DEFAULT_FILES","OutputMarcEditConfigurationFile","",(char *)Temporary2,BUFF_SIZE,itsIniFile);
	if (!*Temporary2)
	{
		ErrorHandler->WriteError("\nNo Output MARC edit configuration file specified in the .ini file");
	}
	// end rev 1.1

	
	// John Hough - added delete
	if (aFileSpec)
		delete aFileSpec;

	aFileSpec = new FILE_SPEC;
	memset(aFileSpec,0,sizeof(FILE_SPEC));
	strcpy(aFileSpec->name,(char *)Temporary);
	xvt_fsys_get_dir(aFileSpec->dir); 
	
	itsMarcDoc->SetMarcFileSpec(aFileSpec);
	
	itsMarcDoc->SetMarcFileFormat(FALSE);//non segmente par defaut
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","IsOutputBlockSegmentedChecked","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary)
	{
		if (!strcmp((char *)Temporary,"TRUE"))
		{
			itsMarcDoc->SetMarcFileFormat(TRUE);
		}
	}

	itsMarcDoc->SetMarcFileBlockSize(2048);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","OutputMarcSizeBlock","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary)
	{
		if (strcmp((char *)Temporary,"2048"))
		{
			itsMarcDoc->SetMarcFileBlockSize((short)atoi((char *)Temporary));
		}
	}
	
	itsMarcDoc->SetMarcFileMinDataFree(5);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","OutputMarcMinDataFree","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary)
	{
		if (strcmp((char *)Temporary,"5"))
		{
			itsMarcDoc->SetMarcFileMinDataFree((short)atoi((char *)Temporary));
		}
	}
	
	itsMarcDoc->SetMarcFilePaddingChar(0x5E);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","OutputMarcPaddingChar","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary)
	{
		if (strcmp((char *)Temporary,"5E"))
		{
			itsMarcDoc->SetMarcFilePaddingChar((char)ToHexa((char *)Temporary));
		}
	}

	itsMarcDoc->SetMarcFileLastBlock(FALSE);
	GetPrivateProfileString("DEFAULT_MARC_ATTRIBUTES","IsOutputLastBlockPaddedChecked","",(char *)Temporary,BUFF_SIZE,itsIniFile);
	if (*Temporary)
	{
		if (!strcmp((char *)Temporary,"TRUE"))
		{
			itsMarcDoc->SetMarcFileLastBlock(TRUE);
		}
	}


	// John Hough - added delete
	if (aFileSpec)
		delete aFileSpec;

	aFileSpec = new FILE_SPEC;
	memset(aFileSpec,0,sizeof(FILE_SPEC));
	strcpy(aFileSpec->name,(char *)Temporary2);
	xvt_fsys_get_dir(aFileSpec->dir); 
	itsMarcDoc->SetConfFileSpec(aFileSpec);         
	
	itsMarcDoc->Open(OUTPUT);






	BOOLEAN ConvOk=TRUE;
	 
	itsExecutionMode      = MODE_CONVERSION;   
	
	itsMarcDoc->GetInputFile()->Reset();
	
	itsMarcOutputOk=0L;
	itsMarcOutputKo=0L;
	
	GetPrivateProfileString("DEFAULT_VALUES","OrdinalNumber","1",(char *)Temporary,BUFF_SIZE,itsIniFile);
	ordinal=atol((char *)Temporary);   


	itsStopRequest = FALSE; 
	itsCloseRequest= FALSE;

	size = (double)itsMarcDoc->GetInputFile()->GetSize();
	
	// John Hough
	g_size = size;
	g_ConvOk = ConvOk;
	g_aFileSpec = aFileSpec;
}




void TUMApplication::Convert(void)
{ 

	double	size;
	int	readcentile;
	int	oldreadcentile;

	BOOLEAN ConvOk=TRUE;

	size = g_size;
	ConvOk = g_ConvOk;

	// John Hough - added if
	if (myrun.GetReadMarcRecordsFromBexFile())
	{
		printf("Size of input file: %ld bytes\n", (long)size);
		printf("Conversion progress:\n");
	}

	long NbTrait=0L;
	oldreadcentile = -1;
	while (itsCloseRequest!=TRUE)
	{ 
		
		if (itsStopRequest!=TRUE && itsMarcDoc->Read()==0)
		{
			{

				ConvOk=TRUE;
				if (itsMarcDoc->Transcode(itsTransDoc)) ConvOk=FALSE;
				if (itsMarcDoc->Convert(itsRuleDoc)) ConvOk=FALSE;
				ordinal++;
				sprintf((char *)Temporary,"%ld",ordinal);
				WritePrivateProfileString("DEFAULT_VALUES","OrdinalNumber",(char *)Temporary,itsIniFile);
				
				// Mise a jour des compteurs
				if (ConvOk==TRUE) 
				{
					++itsMarcOutputOk;
				}
				else
				{
					++itsMarcOutputKo;
				}

				readcentile = (int)(itsMarcDoc->GetInputFile()->GetPos() / size * 100);
				
				// John hough - added if
				if (myrun.GetReadMarcRecordsFromBexFile())
				{
					printf("%d%%   %d input records\r",(int)readcentile, itsMarcOutputOk);
				}
				oldreadcentile = readcentile;

				// Controle du fichier de sortie si un fichier de controle a ete charge           
				if (itsCheckDoc->GetOutputFile())   
				{
					itsCheckDoc->GetOutputFile()->Verify(OUTPUT,itsMarcDoc->GetOutputRecord());   
				}

				DoConvertWrite();  
				 
				if ((ErrorHandler->GetTooManyErrors()) && (ErrorHandler->GetHowManyErrors()>ErrorHandler->GetTooManyErrors()))
					// Nombre d'erreur maxi depasse 
				{
					ErrorHandler->WriteError("\nEND non OK : Maximum errors to be encountered has been reached");
					exit(-1);
				}
			} 
			NbTrait++;
		}
		else
		{ 
			if (itsStopRequest!=TRUE)
			{
				itsCloseRequest = itsStopRequest=TRUE;            
			}
		}  
	}

	
	// John Hough - added if
	if (myrun.GetReadMarcRecordsFromBexFile())
	{
		printf("100%\n");
	}


	// John Hough
	g_NbTrait= NbTrait;

}



void TUMApplication::End(void)
{
	// John Hough
	long NbTrait= g_NbTrait;
	FILE_SPEC *aFileSpec = g_aFileSpec;

	itsMarcDoc->Close(OUTPUT);
	itsMarcDoc->Close(INPUT);
	ErrorHandler->SetMode(BATCH);

	// John Hough - added if
	if (myrun.GetReadMarcRecordsFromBexFile())
	{

		char mess[1000];
		sprintf(mess,"\nEND OK %ld Converted Records : %ld without error and %ld with error(s)",NbTrait,itsMarcOutputOk,itsMarcOutputKo);
		printf("Conversion completed OK\n   with %ld Converted Records : %ld without error and %ld with error(s)\n\n",NbTrait,itsMarcOutputOk,itsMarcOutputKo);
		ErrorHandler->WriteError(mess);
	}
	
	// John Hough - added delete
	if (aFileSpec)
		delete aFileSpec;


	// John Hough - replaced exit with return
	return;
	//exit(0);
}

   
 ///////////////////////////////////////////////////////////////////////////////
 //
 // DoConvertWrite
 //
 ///////////////////////////////////////////////////////////////////////////////   
 int TUMApplication::DoConvertWrite(void)  
 {
	 // On ecrit la notice marc itsMarcDoc->itsOutputRecord dans le fichier de sortie
	 itsMarcDoc->Write();
	 return 0;
 }

int ToHexa(const char* str)
{ 
 int		v=0,
 			i,
 			max=strlen(str)-1,
 			d;
 char		*StrBis=strdup(str);

 for (i=max,d=1;i>=0;--i)
  {
   StrBis[i]=(char)toupper((int)StrBis[i]);
   if (StrBis[i]>='A')
    v+=d*(StrBis[i]-'A'+10);
   else
    v+=d*(StrBis[i]-'0');
   d*=16;
  } 
 free (StrBis); 
 return v;
}


 ///////////////////////////////////////////////////////////////////////////////
 //
 // convert_file_spec_to_str
 //
 ///////////////////////////////////////////////////////////////////////////////   
 void TUMApplication::convert_file_spec_to_str(FILE_SPEC *theFileSpec,char *theString)
 {
  xvt_fsys_convert_dir_to_str(theFileSpec->dir,theString,BUFF_SIZE);
  if (theString[strlen(theString)-1]==SLASH) 
   sprintf(&theString[strlen(theString)],"%s",theFileSpec->name); 
  else  
   sprintf(&theString[strlen(theString)],"%c%s",SLASH,theFileSpec->name);  
 } 
