/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  trnsfile.cpp
 *
 *  implements a class to manage transcoding file
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h> 
#include "trnsfile.h"
#include "error.h"   
#include "tmarcrec.h"              
   
///////////////////////////////////////////////////////////////////////////////
//
// TTransFile
//
///////////////////////////////////////////////////////////////////////////////   
TTransFile::TTransFile( FILE_SPEC *FileSpec)
		   :TFile( FileSpec), semi()
 {
   itsFileInfo=new FILE_SPEC;
   DefaultValue=NULL;
   if (itsFileInfo)
    memcpy(itsFileInfo,FileSpec,sizeof(FILE_SPEC));
 }
   
///////////////////////////////////////////////////////////////////////////////
//
// ~TTransFile
//
///////////////////////////////////////////////////////////////////////////////   
TTransFile::~TTransFile() 
 {
  if (itsFileInfo) delete itsFileInfo;
  if (DefaultValue) free(DefaultValue);
 }  
  
///////////////////////////////////////////////////////////////////////////////
//
// Open
//
///////////////////////////////////////////////////////////////////////////////   
int TTransFile::Open() 
{
 char 			aLine[BUFF_SIZE+1];
 char 			aComment[2*BUFF_SIZE];
 char* 			InChar;
 char*			OutChar;
 char			InListe[BUFF_SIZE+1];
 char			OutListe[BUFF_SIZE+1];
 int			Line;			
 FILE_SPEC		aSpec;

 // Ouverture du fichier
 if (TFile::Open())
  return 1;

 // Lecture de chaque ligne du fichier
 while( NextLine( aLine, BUFF_SIZE, aComment, BUFF_SIZE, &aSpec, &Line )==0)
{
    // Si la ligne n'est pas vide, la traiter
    if (*aLine)
     {
       // Decoupage sur le |
       InChar=OutChar=aLine;   
       while(*OutChar && *OutChar!='|') ++OutChar;
       if (*OutChar) ++OutChar;

       // Test de la valeur par defaut
       if (memcmp(InChar,"#DEFAULT",8)==0)
        { 
         GetValues(OutChar,OutListe);
         DefaultValue=(void*)strdup(OutListe);
        }
       else
        {
         // Affectation des valeurs
         GetValues(InChar,InListe);
         GetValues(OutChar,OutListe);
        
         // Creation de l'arbre lexicographique
         if (*InListe)
		 //{
		  // John Hough - added ptr in place of (unsigned char*)strdup(OutListe)
		  
		  //char *ptr = strdup(OutListe);
          //insere((unsigned char*)InListe, (unsigned char*)ptr);

		  //if (ptr)
			//  free (ptr);
		  
          insere((unsigned char*)InListe, (unsigned char*)strdup(OutListe));

		 //}
        }
     }
  }
 
 // On referme le fichier
 TFile::Close(); 
 return 0;
}



///////////////////////////////////////////////////////////////////////////////
//
// Convert
//
// Cette methode convertit une notice en une autre conformement au fichier de
// transco charg
//
///////////////////////////////////////////////////////////////////////////////   
int TTransFile::Convert( TMarcRecord* MarcIn, TMarcRecord* MarcOut )
 {
  TMarcField* In;
  TMarcField* Out;
                   
  // Recuperation de la cle de la notice dans Temporary2 
  strcpy((char *)Temporary2,MarcIn->GetFirstField()->GetLib());
                   
  // On commence par effectuer la recopie (sans transco) du label
  MarcOut->SetLabel(MarcIn->GetLabel());
  if (!MarcOut->GetLabel())
   { 
    sprintf((char *)Temporary,"Notice '%s' : label '%s'",(char *)Temporary2,MarcIn->GetLabel());   
    ErrorHandler->SetErrorD( 3000, ERROR, (char *)Temporary ); 
   }
   
  // On transcode ensuite chaque champ
  In=MarcIn->GetFirstField();
  Out=MarcOut->GetFirstField();
  while( In!=NULL )
   {
    // On cree un nouveau champ en sortie
    if (Out==NULL)
     {
      MarcOut->SetFirstField(new TMarcField);
      Out=MarcOut->GetFirstField();
     }
    else
     {
      Out->SetNextField(new TMarcField);
      Out=Out->GetNextField();
     }

    // On recopie le tag et les indicateurs du champ en entree
    Out->SetTag(In->GetTag());
    Out->SetIndicators(In->GetIndicators());

    // On transcode le libelle
    Out->SetLib( Transcode(In->GetLib(), (char*)Temporary, (char *)Temporary2, In->GetTag()));
    if (!Out->GetLib())
     {
      sprintf((char *)Temporary,"Notice '%s' : field '%s'",(char *)Temporary2,In->GetTag());     
      ErrorHandler->SetErrorD( 3000, ERROR, (char *)Temporary );    
     } 
    
    // On passe au champ suivant
    In=In->GetNextField();
   }

  return ErrorHandler->GetErrorCode();
 }


///////////////////////////////////////////////////////////////////////////////
//
// Transcode
//
// Cette methode transcode une chaine de caractere en une autre, conformement
// a la table chargee - inspire de ttrans.cc
//
///////////////////////////////////////////////////////////////////////////////   
char*
TTransFile::Transcode( char* In, char* Out, char *Notice, char *Field )
 {
  char *s = Out;
  char *c = In;
  void *r = NULL;
  
  while(*c != '\0')
   {
    int iaux;
    r  = (char *)cherche((unsigned char*)c,&iaux);
    if((iaux == 0) || (r == NULL))
     {
      if (DefaultValue)
       {
        strcpy(Out,(char*)DefaultValue);
        return Out;
       }


      // aucune chaine de sustitution n'a ete trouvee, on conserve la valeur
      // en signalant l'erreur
      char tmp[2000];
      if (Notice && Field)
       sprintf(tmp, "Notice '%s' : field '%s' ( Unknown character '%c' (\\%x) ) : table '%s'",Notice,Field,(unsigned char)*c,(unsigned char)*c,itsFileInfo->name);
      else
       sprintf(tmp, "( Unknown character '%c' (\\%x) ) : table '%s'",(unsigned char)*c,(unsigned char)*c,itsFileInfo->name);
      ErrorHandler->SetErrorD( 3001, ERROR, tmp );
      *s = *c;
      s++;
      c++;
     }
    else
     {
      /* on a trouve une correspondance */
      strcpy(s,(char *)r);
      c+=iaux;
      s+= strlen((char *)r);
     }
   }
  *s = '\0';
  return(Out);
 }

///////////////////////////////////////////////////////////////////////////////
//
// GetValues
//
///////////////////////////////////////////////////////////////////////////////   
int
TTransFile::GetValues(char* Str, char* Liste)
 {
  char*c = Str;
  char*e = Liste;
  int fin=0,iaux;
  
  while(!fin)
   {
    // On saute les caracteres non representatifs
    while((*c == ' ')||(*c == '\t')) c++;
      
    switch(*c)
     {
      case 0:
      case '\n':
      case '|':
	fin =1;
	*e ='\0';
	break;

      case '0':
	/* le prochain caractere est represente par un code hexa */
	sscanf(c,"%i",&iaux);
	*e = (char) iaux;
	e++;
	break;

      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
	sscanf(c,"%d",&iaux);
	*e = (char) iaux;
	e++;
	break;

      default:
	sscanf(c,"%c",e);
	e++;
	break;
      }
      /* on avance jusqu'au prochain espace */
      while((*c != '\0') && (*c != ' ')&& (*c != '\t')) c++;
  }
 return 0;
}

