/*  
 *  USEMARCON Software - Command Line version
 *  Copyright The British Library, The USEMarcon Consortium, 1995-2000
 *  Adapted by Crossnet Systems Limited - British Library Contract No. BSDS 851
 *  
 *  File:  truledoc.cpp
 *
 *  implements a base class to manage file rule files
 *
 *  NOTE:  IN NO WAY WHATSOEVER SHOULD THIS FILE BE USED IN THE EARLIER
 *         VERSIONS OF USEMARCON SOFTWARE.
 *
 */

#include <afx.h>
#include "error.h"

#include "defines.h"
#include "truledoc.h"
#include "tmpplctn.h"
#include "xvtstuff.h"



extern TUMApplication *g_theApplication;



///////////////////////////////////////////////////////////////////////////////
//
// Document class constructor
//
// Create and initialize the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TRuleDoc::TRuleDoc( ) 
{

 itsFile			= NULL;
 itsRuleSpec		= NULL;
 itsInputFormat		= NULL;
 itsOutputFormat	= NULL;
 itsXVTFilePointer  = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//
// Document class destructor
//
// Delete the internal data objects for this class
//
///////////////////////////////////////////////////////////////////////////////
TRuleDoc::~TRuleDoc( void )
{        
 if (itsFile)			{ delete itsFile; 			itsFile				= NULL; }
 if (itsInputFormat)	{ free (itsInputFormat);	itsInputFormat		= NULL; }
 if (itsOutputFormat)	{ free (itsOutputFormat);	itsOutputFormat		= NULL; } 

  // John Hough - added delete
 if (itsXVTFilePointer)
	 delete itsXVTFilePointer;


}


  

///////////////////////////////////////////////////////////////////////////////
//
// Open
//
// Called by DoOpen().  Do the actual work of restoring the object from
// a data source.
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN TRuleDoc::Open(void)
{
 // Add code here to set internal data members of the class based
 // on information read from some data source
 if ((itsFile=new TRuleFile(itsXVTFilePointer))==NULL)
  {
   ErrorHandler->SetError(9001,ERROR);
   return FALSE;
  } 
  
 if (itsFile->OpenRuleFile())
  {
   // Rule file loading failure
   delete itsFile;
   itsFile = NULL;
   return FALSE;
  } 
 return TRUE; 
}



///////////////////////////////////////////////////////////////////////////////
//
// DoOpen
//
// This method is automatically called when the File Save menu option is
// selected.  Initialize the contents of this document object from the file
// path in the internal FILE_SPEC data member.  Put up the File Open dialog
// if no file name has been assigned yet.
//
///////////////////////////////////////////////////////////////////////////////
BOOLEAN TRuleDoc::DoOpen(void)
{
	return 0;
}  
            
         
///////////////////////////////////////////////////////////////////////////////
//
// OpenRuleFile
//
///////////////////////////////////////////////////////////////////////////////       
BOOLEAN TRuleDoc::OpenRuleFile(char *DefaultFile)
{
 FILE_SPEC		*NewFileSpec;
 int			Result;
 char			*ItemPointer;
                  
 NewFileSpec = new FILE_SPEC;

 if (DefaultFile)
 // The Rule File has already been selectionned
  {              
   if ((ItemPointer=(char *)strrchr(DefaultFile,SLASH))!=NULL)
    {
     strcpy(NewFileSpec->name,ItemPointer+1);      
     NewFileSpec->type[0]=0;
     *ItemPointer = NULL;
     xvt_fsys_convert_str_to_dir(DefaultFile,NewFileSpec->dir);
    }                                                           
   else
    { 
     strcpy(NewFileSpec->name,DefaultFile);
     NewFileSpec->type[0]=0;            
     xvt_fsys_get_dir(NewFileSpec->dir);
    }
   Result=FL_OK;
  }
  
 switch(Result)
  { 
   case FL_OK:						// a rule file has been selected
                 
    if (itsXVTFilePointer)
     delete itsXVTFilePointer;
    itsXVTFilePointer = NewFileSpec;

    if (Open()) 					// Open the selected Rule file
     {
      itsRuleSpec = itsXVTFilePointer;
      return TRUE;
     }
    else
     return FALSE;
    break;
     
   case FL_BAD:
    return FALSE;
    break;
    
   case FL_CANCEL:
   default:
    return TRUE;
  }
} 


