// John Hough - the objects I've added to the application

#ifndef objectlist_h
#define objectlist_h

#include "defines.h"
#include "cstartup.h"


#ifndef OBJECT_DEFS 
#define EXTERN extern
#else
#define EXTERN
#endif

#define ILLEGAL_LENGTH_RECORD "Illegal length marc record"

// An object used for maintenance as a means
// to pass data into the rest of the application
class CDetails {

private:
	bool m_fReadMarcRecordsFromBexFile;
	char *m_pcRecord;
	int m_iLength;
	bool m_fIniDetailsLoaded;
	char *m_pcIniFileName;

public:
	CDetails() 
	{
		m_pcRecord = NULL;
		m_iLength = 0;

		m_fIniDetailsLoaded = false;
		m_pcIniFileName = NULL;
	};
	
	// Destructor called when the object goes out of scope
	~CDetails() 
	{
		DeleteData();

		// no destructor for m_pcIniFileName because a non alloced string constant may be used
	}


	void DeleteData(void)
	{
		if (m_pcRecord)
		{
			delete m_pcRecord;
			m_pcRecord = NULL;
		}
	}



	void SetReadMarcRecordsFromBexFile(bool fValue)
	{
		m_fReadMarcRecordsFromBexFile = fValue;
	} 

	bool GetReadMarcRecordsFromBexFile(void)
	{
		return m_fReadMarcRecordsFromBexFile;
	} 

	void SetMarcRecord(char *pcRecord, int iLength)
	{
		SetMarcLength(iLength);
		SetMarcData(pcRecord);
	}
	

	void GetMarcRecord(char *&pcRecord, int &iLength)
	{
		pcRecord = m_pcRecord;
		iLength = m_iLength;
	}


	void SetInputMarcRecord(char *pcRecord, int iLength)
	{
		SetMarcRecord(pcRecord, iLength);
	}
	

	void GetInputMarcRecord(char *&pcRecord, int &iLength)
	{
		GetMarcRecord(pcRecord, iLength);
	}


	void SetOutputMarcRecord(char *pcRecord, int iLength)
	{
		SetMarcRecord(pcRecord, iLength);
	}
	

	void GetOutputMarcRecord(char *&pcRecord, int &iLength)
	{
		GetMarcRecord(pcRecord, iLength);
	}


	void SetMarcLength(int iLength)
	{
		m_iLength = iLength; 
	}


	void SetMarcData(char *pcRecord)
	{
		// Remove any old instance
		DeleteData();

		if (m_iLength <= TBUF)
		{
			m_pcRecord = new char[m_iLength + 1];
			memset(m_pcRecord, 0, sizeof(m_pcRecord));
			memcpy(m_pcRecord, pcRecord, m_iLength);
		}
		else
		{
			pcRecord = new char[strlen(ILLEGAL_LENGTH_RECORD) + 1];
			strcpy(m_pcRecord, ILLEGAL_LENGTH_RECORD);
			m_iLength = 0;
		}
	}


	void SetIniDetailsLoaded(bool fIniDetailsLoaded)
	{
		m_fIniDetailsLoaded = fIniDetailsLoaded;
	}

	bool GetIniDetailsLoaded()
	{
		return m_fIniDetailsLoaded;
	}


	void SetIniFileName(char *pcIniFile)
	{
		m_pcIniFileName = pcIniFile;
	}

	void start(void)
	{
		int argc_copy = 1;
		char *argv_copy[2] = {"usemarcon", NULL};

		if (m_pcIniFileName) // if .ini file specified
		{
			argc_copy = 2;
			argv_copy[1] = m_pcIniFileName;
		}

		begin(argc_copy, argv_copy);
	}
};



EXTERN CDetails myrun;

#endif
